#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from networking_plumgrid._i18n import _LI
from networking_plumgrid.neutron.plugins.common import \
    constants
from networking_plumgrid.neutron.plugins.common import \
    policy_exceptions as p_excep
from neutron.api.v2 import attributes
from neutron.db import common_db_mixin
from neutron.db import model_base
from neutron.db import models_v2
from oslo_log import log as logging
import sqlalchemy as sa
from sqlalchemy import orm
from sqlalchemy.orm import exc

LOG = logging.getLogger(__name__)


class PolicyService(model_base.BASEV2, models_v2.HasId,
                    models_v2.HasTenant):
    """DB definition for PLUMgrid policy service object"""

    __tablename__ = "pg_policy_services"

    name = sa.Column(sa.String(attributes.NAME_MAX_LEN))
    description = sa.Column(sa.String(255))


class PolicyServicePortBinding(model_base.BASEV2):
    """
    DB definition for ports required by PLUMgrid policy service.
    Represents binding between neutron ports
    and policy services.
    """
    __tablename__ = "pg_policy_service_port_binding"

    port_id = sa.Column(sa.String(36),
                        sa.ForeignKey("ports.id",
                                      ondelete='CASCADE'),
                        primary_key=True)
    policy_service_id = sa.Column(sa.String(36),
                                  sa.ForeignKey("pg_policy_services.id"),
                                  primary_key=True)

    direction = sa.Column(sa.Enum(constants.DIRECTION_INGRESS,
                                  constants.DIRECTION_EGRESS,
                                  constants.BIDIRECTIONAL), primary_key=True)

    # Add a relationship to the Port model in order to instruct SQLAlchemy to
    # eagerly load policy service bindings
    ps_ports = orm.relationship(models_v2.Port,
                             backref=orm.backref("ps_port",
                                                 lazy='joined',
                                                 cascade="delete"))
    policy_service = orm.relationship(PolicyService,
                  backref=orm.backref("psp_binding",
                                  lazy="joined", cascade="all,delete-orphan"),
    primaryjoin="PolicyService.id==PolicyServicePortBinding.policy_service_id")


class PolicyServiceMixin(common_db_mixin.CommonDbMixin):
    def create_policy_service(self, context, policy_service):
        """
        creates a policy with initial set of ports
        Args:
             policy_service:
                   JSON object with policy service attributes
                   name : display name policy service
                   tenant_id : tenant uuid
                   id : policy service uuid
                   description : description of policy service
                   ingress_ports : List of ingress ports to be classified into
                                   policy service
                   egress_ports: List of egress ports to be classified into
                                 policy service
                   bidirectional_ports: List of bidirectional ports to be
                                        classified into policy service
        """
        ps = policy_service["policy_service"]
        #  check availability of ports
        if "ingress_ports" in ps and ps["ingress_ports"]:
            self._check_port_ps_association(context, ps["ingress_ports"])
        if "egress_ports" in ps and ps["egress_ports"]:
            self._check_port_ps_association(context, ps["egress_ports"])
        if "bidirectional_ports" in ps and ps["bidirectional_ports"]:
            self._check_port_ps_association(context,
                                            ps["bidirectional_ports"])
        LOG.info(_LI("Creating policy service %(name)s."),
                 {'name': ps["name"]})
        with context.session.begin(subtransactions=True):
            ps_db = PolicyService(tenant_id=ps["tenant_id"],
                                  name=ps["name"],
                                  description=ps["description"])
            context.session.add(ps_db)
        with context.session.begin(subtransactions=True):
            for ingress_port in ps["ingress_ports"]:
                ps_ingress_port_db = PolicyServicePortBinding(
                                         port_id=ingress_port["id"],
                                         policy_service_id=ps_db.id,
                                         direction=constants.DIRECTION_INGRESS)
                context.session.add(ps_ingress_port_db)
            for egress_port in ps["egress_ports"]:
                ps_egress_port_db = PolicyServicePortBinding(
                                         port_id=egress_port["id"],
                                         policy_service_id=ps_db.id,
                                         direction=constants.DIRECTION_EGRESS)
                context.session.add(ps_egress_port_db)
            for bidirect_port in ps["bidirectional_ports"]:
                ps_bidirect_port_db = PolicyServicePortBinding(
                                         port_id=bidirect_port["id"],
                                         policy_service_id=ps_db.id,
                                         direction=constants.BIDIRECTIONAL)
                context.session.add(ps_bidirect_port_db)
        return self._make_ps_dict(ps_db)

    def get_policy_service(self, context, ps_id, fields=None):
        """
        Gets an existing policy service
        Args:
             ps_id = uuid of the policy service being requested
        """
        try:
            query = self._model_query(context, PolicyService)
            ps_db = query.filter_by(id=ps_id).one()
        except exc.NoResultFound:
            raise p_excep.NoPolicyServiceFound(id=ps_id)
        return self._make_ps_dict(ps_db, fields)

    def get_policy_services(self, context, filters=None, fields=None,
             sorts=None, limit=None, marker=None, page_reverse=None):
        """
        Gets the list of all the existing policy services
        """
        return self._get_collection(context, PolicyService,
                                    self._make_ps_dict, filters=filters,
                                    sorts=sorts, limit=limit,
                                    marker_obj=marker, fields=fields,
                                    page_reverse=page_reverse)

    def delete_policy_service(self, context, ps_id):
        """
        Deletes an existing policy service
        Args:
             ps_id = uuid of the policy service being deleted
        """
        try:
            query = context.session.query(PolicyService)
            ps_db = query.filter_by(id=ps_id).first()
            self._check_ps_in_use(context, ps_db)
        except exc.NoResultFound:
            raise p_excep.NoPolicyServiceFound(id=ps_id)
        with context.session.begin(subtransactions=True):
            context.session.delete(ps_db)

    def update_policy_service(self, context, ps_id, policy_service):
        """
        Updates an existing policy
        Args:
             ps_id:
                   uuid of the policy service being updated
             policy_service:
                   JSON with updated attributes of the policy service
                   name : name of policy service
                   tenant_id : tenant uuid
                   id : uuid of policy service
                   description : description of policy service
                   ingress_ports : List of ingress ports of policy service
                   egress_ports: List of egress ports of policy service
                   add_ingress_ports: List of ingress ports to be added to
                                      policy service
                   remove_ingress_ports: List of ingress ports to be removed
                                         from policy service
                   add_egress_ports: List of egress ports to be added to
                                     policy service
                   remove_egress_ports: List of egress ports to be removed
                                        from policy service
        """
        ps = policy_service["policy_service"]
        if not ps:
            raise p_excep.UpdateParametersRequired()
        with context.session.begin(subtransactions=True):
            try:
                query = self._model_query(context, PolicyService)
                ps_db = query.filter_by(id=ps_id).one()
            except exc.NoResultFound:
                raise p_excep.NoPolicyServiceFound(id=ps_id)
            if 'name' in ps:
                ps_db.update({"name": ps["name"]})

            if 'description' in ps:
                ps_db.update({"description": ps["description"]})

            if 'add_ingress_ports' in ps:
                self._check_port_ps_association(context,
                                                ps["add_ingress_ports"])
                for ingress_port in ps["add_ingress_ports"]:
                    ps_ingress_port_db = PolicyServicePortBinding(
                                         port_id=ingress_port["id"],
                                         policy_service_id=ps_db.id,
                                         direction=constants.DIRECTION_INGRESS)

                    context.session.add(ps_ingress_port_db)
            if 'add_egress_ports' in ps:
                self._check_port_ps_association(context,
                                                ps["add_egress_ports"])
                for egress_port in ps["add_egress_ports"]:
                    ps_egress_port_db = PolicyServicePortBinding(
                                         port_id=egress_port["id"],
                                         policy_service_id=ps_db.id,
                                         direction=constants.DIRECTION_EGRESS)

                    context.session.add(ps_egress_port_db)
            if 'remove_ingress_ports' in ps:
                for ingress_port in ps["remove_ingress_ports"]:
                    query = self._model_query(context,
                                              PolicyServicePortBinding)
                    query.filter_by(port_id=ingress_port["id"],
                                    policy_service_id=ps_db.id).delete()
            if 'remove_egress_ports' in ps:
                for egress_port in ps["remove_egress_ports"]:
                    query = self._model_query(context,
                                              PolicyServicePortBinding)
                    query.filter_by(port_id=egress_port["id"],
                                    policy_service_id=ps_db.id).delete()
            if 'add_bidirectional_ports' in ps:
                self._check_port_ps_association(context,
                                                ps["add_bidirectional_ports"])
                for bidirect_port in ps["add_bidirectional_ports"]:
                    ps_bidirect_port_db = PolicyServicePortBinding(
                                             port_id=bidirect_port["id"],
                                             policy_service_id=ps_db.id,
                                             direction=constants.BIDIRECTIONAL)

                    context.session.add(ps_bidirect_port_db)
            if 'remove_bidirectional_ports' in ps:
                for bidirect_port in ps["remove_bidirectional_ports"]:
                    query = self._model_query(context,
                                              PolicyServicePortBinding)
                    query.filter_by(port_id=bidirect_port["id"],
                                    policy_service_id=ps_db.id).delete()
            query = self._model_query(context, PolicyServicePortBinding)
            ps_db.psp_binding = query.filter_by(policy_service_id=ps_id).all()
        return self._make_ps_dict(ps_db)

    def _check_port_ps_association(self, context, ports):
        for port in ports:
            query = self._model_query(context, PolicyServicePortBinding)
            port_id = port["id"]
            try:
                port_in_use = query.filter_by(port_id=port_id).one()
                if port_in_use:
                    raise p_excep.PortAlreadyInUsePolicyService(port=port_id,
                              ps=port_in_use.policy_service_id)
            except exc.NoResultFound:
                pass

    def _make_ps_dict(self, ps, fields=None):
        ingress_ports = []
        egress_ports = []
        bidirect_ports = []
        for port in ps.psp_binding:
            if port.direction == constants.DIRECTION_INGRESS:
                ingress_ports.append({"id": port.port_id})
            elif port.direction == constants.DIRECTION_EGRESS:
                egress_ports.append({"id": port.port_id})
            elif port.direction == constants.BIDIRECTIONAL:
                bidirect_ports.append({"id": port.port_id})
            else:
                raise p_excep.InvalidPolicyServiceConfiguration()
        ps_dict = {"id": ps.id,
                   "name": ps.name,
                   "ingress_ports": ingress_ports,
                   "egress_ports": egress_ports,
                   "bidirectional_ports": bidirect_ports,
                   "description": ps.description,
                   "tenant_id": ps.tenant_id}
        return self._fields(ps_dict, fields)

    def _check_ps_in_use(self, context, ps_db):
        ports = []
        for port in ps_db.psp_binding:
            port_id = port.port_id
            query = self._model_query(context, PolicyServicePortBinding)
            try:
                port_in_use = query.filter_by(port_id=port_id).one()
                if port_in_use:
                    ports.append(port_id)
            except exc.NoResultFound:
                pass
        if ports:
            raise p_excep.PolicyServiceInUse(id=ps_db.id, port=str(ports))



# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""active_standby_pap

Revision ID: 85d77526e1f6
Revises: 351c4f5710e7
Create Date: 2016-08-14 11:07:32.564472

"""

# revision identifiers, used by Alembic.
revision = '85d77526e1f6'
down_revision = '351c4f5710e7'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column(
        'pg_physical_attachment_points',
        sa.Column('active_standby', sa.Boolean(), nullable=False),
    )



# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from networking_plumgrid._i18n import _
from neutronclient.common import extension


class TransitDomain(extension.NeutronClientExtension):
    resource = 'transit_domain'
    resource_plural = 'transit_domains'
    path = 'transit-domains'
    object_path = '/%s' % path
    resource_path = '/%s/%%s' % path
    versions = ['2.0']


def args2body(self, parsed_args):
    try:
        if parsed_args.name:
            tvd_name = parsed_args.name
            body = {'transit_domain': {'name': tvd_name}}
        else:
            body = {'transit_domain': {}}
        return body
    except KeyError as err:
        raise Exception("KeyError: " + str(err))


class TransitDomainCreate(extension.ClientExtensionCreate,
                          TransitDomain):
    """Create a transit domain."""

    shell_command = 'transit-domain-create'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name', metavar='<TRANSIT-DOMAIN-NAME>',
            help=_('Descriptive name for transit domain.'))

    def args2body(self, parsed_args):
        body = args2body(self, parsed_args)
        if parsed_args.tenant_id:
            (body['transit_domain']
                 ['tenant_id']) = parsed_args.tenant_id
        return body


class TransitDomainList(extension.ClientExtensionList,
                        TransitDomain):
    """List transit domains"""

    shell_command = 'transit-domain-list'
    list_columns = ['id', 'name']
    pagination_support = True
    sorting_support = True


class TransitDomainShow(extension.ClientExtensionShow,
                        TransitDomain):
    """Show information of a given transit domain"""

    shell_command = 'transit-domain-show'


class TransitDomainDelete(extension.ClientExtensionDelete,
                          TransitDomain):
    """Delete a given transit domain"""

    shell_command = 'transit-domain-delete'


class TransitDomainUpdate(extension.ClientExtensionUpdate,
                          TransitDomain):
    """Update a given transit domain"""

    shell_command = 'transit-domain-update'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--name', metavar='name',
            help=_('Descriptive name for transit domain'))

    def args2body(self, parsed_args):
        body = {'transit_domain': {'name': parsed_args.name}}
        return body



# Copyright 2016 PLUMgrid, Inc. All Rights Reserved.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import mock
from networking_plumgrid.neutron.tests.unit.cli import test_cli20
from networking_plumgrid.neutronclient.policy import (
    endpoint_group as epg)
from neutronclient import shell
import sys


class CLITestV20ExtensionEndpointGroupJSON(test_cli20.CLITestV20Base):
    def setUp(self):
        # need to mock before super because extensions loaded on instantiation
        self._mock_extension_loading()
        super(CLITestV20ExtensionEndpointGroupJSON,
              self).setUp(plurals={'tags': 'tag'})

    def _mock_extension_loading(self):
        ext_pkg = 'neutronclient.common.extension'
        contrib = mock.patch(ext_pkg + '._discover_via_entry_points').start()
        contrib.return_value = [("endpoint_group", epg)]
        return contrib

    def test_ext_cmd_loaded(self):
        """Tests endpoint group commands loaded."""
        shell.NeutronShell('2.0')
        ext_cmd = {'endpoint-group-list':
                   epg.EndpointGroupList,
                   'endpoint-group-create':
                   epg.EndpointGroupCreate,
                   'endpoint-group-update':
                   epg.EndpointGroupUpdate,
                   'endpoint-group-delete':
                   epg.EndpointGroupDelete,
                   'endpoint-group-show':
                   epg.EndpointGroupShow}
        self.assertDictContainsSubset(ext_cmd, shell.COMMANDS['2.0'])

    def test_create_endpoint_group_name(self):
        # Create endpoint group: epg1.
        resource = 'endpoint_group'
        cmd = epg.EndpointGroupCreate(test_cli20.MyApp(
                                      sys.stdout), None)
        name = 'epg1'
        args = [name]
        position_names = ['name']
        position_values = [name]
        self._test_create_resource(resource, cmd, name, 'myid', args,
                                   position_names, position_values)

    def test_create_endpoint_group_description(self):
        # Create endpoint group: epg1.
        resource = 'endpoint_group'
        cmd = epg.EndpointGroupCreate(test_cli20.MyApp(
                                      sys.stdout), None)
        name = 'epg1'
        description = 'epg desc'
        args = [name, '--description', description]
        position_names = ['name', 'description']
        position_values = [name, description]
        self._test_create_resource(resource, cmd, name, 'myid', args,
                                   position_names, position_values)

    def test_create_endpoint_group_tag(self):
        # Create endpoint group: epg1.
        resource = 'endpoint_group'
        cmd = epg.EndpointGroupCreate(test_cli20.MyApp(
                                      sys.stdout), None)
        name = 'epg1'
        tag = 'tagid'
        args = [name, '--tag', tag]
        position_names = ['name', 'policy_tag_id']
        position_values = [name, tag]
        self._test_create_resource(resource, cmd, name, 'myid', args,
                                   position_names, position_values)

    def test_update_endpoint_group_name(self):
        # Update endpointgroup: myid --name
        resource = 'endpoint_group'
        cmd = epg.EndpointGroupUpdate(test_cli20.MyApp(
                                      sys.stdout), None)
        args = ['myid', '--name', 'myepg']
        values = {'name': 'myepg'}
        self._test_update_resource(resource, cmd, 'myid',
                                   args, values)

    def test_update_endpoint_group_desc(self):
        # Update endpointgroup: myid --description
        resource = 'endpoint_group'
        cmd = epg.EndpointGroupUpdate(test_cli20.MyApp(
                                      sys.stdout), None)
        args = ['myid', '--description', 'myepg desc']
        values = {'description': 'myepg desc'}
        self._test_update_resource(resource, cmd, 'myid',
                                   args, values)

    def test_update_endpoint_group_add_tag(self):
        # Update endpointgroup: myid --add-tag
        resource = 'endpoint_group'
        cmd = epg.EndpointGroupUpdate(test_cli20.MyApp(
                                      sys.stdout), None)
        args = ['myid', '--add-tag', 'tagid']
        values = {'add_tag': 'tagid'}
        self._test_update_resource(resource, cmd, 'myid',
                                   args, values)

    def test_update_endpoint_group_remove_tag(self):
        # Update endpointgroup: myid --remove-tag
        resource = 'endpoint_group'
        cmd = epg.EndpointGroupUpdate(test_cli20.MyApp(
                                      sys.stdout), None)
        args = ['myid', '--remove-tag', 'tagid']
        values = {'remove_tag': 'tagid'}
        self._test_update_resource(resource, cmd, 'myid',
                                   args, values)

    def test_list_endpoint_groups(self):
        # list endpointgroups: -D.
        resources = "endpoint_groups"
        cmd = epg.EndpointGroupList(test_cli20.MyApp(
                                    sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_list_endpoint_groups_sort(self):
        # list endpointgroups:
        # --sort-key name --sort-key id --sort-key asc --sort-key desc
        resources = "endpoint_groups"
        cmd = epg.EndpointGroupList(test_cli20.MyApp(
                                    sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_endpoint_groups_limit(self):
        # list endpointgroups: -P.
        resources = "endpoint_groups"
        cmd = epg.EndpointGroupList(test_cli20.MyApp(
                                    sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_delete_endpoint_group(self):
        # Delete endpointgroup: myid.
        resource = "endpoint_group"
        cmd = epg.EndpointGroupDelete(test_cli20.MyApp(
                                      sys.stdout), None)
        my_id = 'myid'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)

    def test_show_endpoint_group(self):
        # Show endpointgroup: --fields id.
        resource = 'endpoint_group'
        cmd = epg.EndpointGroupShow(test_cli20.MyApp(
                                    sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args,
                                 ['id', 'name'])



# Copyright 2016 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from networking_plumgrid._i18n import _
from neutronclient.common import extension


class PolicyRule(extension.NeutronClientExtension):
    resource = 'policy_rule'
    resource_plural = 'policy_rules'
    path = 'policy-rules'
    object_path = '/%s' % path
    resource_path = '/%s/%%s' % path
    versions = ['2.0']


def add_known_arguments(self, parser):
        parser.add_argument(
            '--src-grp', dest='src_grp',
            help=_('Source endpoint group or security group '
                   'name or ID to apply policy.'))
        parser.add_argument(
            '--dest-grp', dest='dst_grp',
            help=_('Destination endpoint group OR security group '
                   'name or ID to apply policy.'))
        parser.add_argument(
            '--protocol', dest='protocol',
            help=_('Protocol of packet. Allowed values are: '
                   '[any, icmp, tcp, udp]'))
        parser.add_argument(
            '--src-port-range', metavar='e.g. 1-65535',
            dest='src_port_range',
            help=_('Source port range for policy. Not supported for ICMP'))
        parser.add_argument(
            '--dest-port-range', metavar='e.g. 1-65535',
            dest='dst_port_range',
            help=_('Destination port range. Not supported for ICMP.'))
        parser.add_argument(
            '--action', dest='action',
            help=_('Action to be performed by the policy. Allowed'
                   ' values are: [copy, allow]'))
        parser.add_argument(
            '--action-target', metavar='<service_name/id or '
            'or tenant_id:service_name/uuid',
            dest='action_target',
            help=_('Action target for policy rule  i.e. policy service'
                   ' or tenant_id:policy service'))
        parser.add_argument(
            '--sgt', action='store_true',
            help=_('Apply source endpoint group tag to policy rule'))
        parser.add_argument(
            '--dgt', action='store_true',
            help=_('Apply destination endpoint group tag to policy rule'))
        parser.add_argument(
            '--tag', metavar='POLICY_TAG',
            dest='tag',
            help=_('Global policy tag to be applied to policy rule'))


def args2body(self, parsed_args):
    try:
        if parsed_args.name:
            pr_name = parsed_args.name
            body = {'policy_rule': {'name': pr_name}}
        else:
            body = {'policy_rule': {}}
        if parsed_args.action:
            if (str(parsed_args.action).lower() == 'copy' or
                str(parsed_args.action).lower() == 'allow'):
                body['policy_rule']['action'] = parsed_args.action
            else:
                raise Exception("Supported values for action are:"
                                " [allow, copy]")
        if parsed_args.src_grp:
            body['policy_rule']['src_grp'] = parsed_args.src_grp
        if parsed_args.dst_grp:
            body['policy_rule']['dst_grp'] = parsed_args.dst_grp
        if parsed_args.src_port_range:
            body['policy_rule']['src_port_range'] \
                = parsed_args.src_port_range
        if parsed_args.dst_port_range:
            body['policy_rule']['dst_port_range'] \
                = parsed_args.dst_port_range
        if parsed_args.protocol:
            supported_protocol = ['any', 'tcp', 'udp', 'icmp']
            if (str(parsed_args.protocol).lower() in supported_protocol):
                body['policy_rule']['protocol'] = parsed_args.protocol
            else:
                raise Exception("Supported values for protocol are:"
                                " [any, icmp, tcp, udp]")
        if (str(parsed_args.action).lower() == 'allow' and
            parsed_args.action_target):
            raise Exception("Target can only be specified with the"
                            " following action(s): ['copy']")
        if (str(parsed_args.action).lower() == 'copy'
            and not parsed_args.action_target):
            raise Exception("Action Target should be specified with "
                            "the following action(s): ['copy']")
        if (parsed_args.action_target and
            str(parsed_args.action).lower() != 'copy'):
            raise Exception("Action Target should be specified with "
                            "the following action(s): ['copy']")
        if parsed_args.action_target:
            if(len(parsed_args.action_target.split(":")) > 2):
                raise Exception("Invalid format for action target. "
                                "Help: <service_name/id or "
                                "or tenant_id:service_name/uuid>")
            body['policy_rule']['action_target'] \
                = parsed_args.action_target
        if parsed_args.sgt and parsed_args.dgt:
            raise Exception("You cannot enable source and destination "
                            "policy tag for a policy_rule.")
        if (parsed_args.tag and (parsed_args.sgt or parsed_args.dgt)):
            raise Exception("You cannot enable global policy tag with sgt or "
                            "dgt for a policy_rule.")

        if ((parsed_args.src_port_range or parsed_args.dst_port_range) and
            not parsed_args.protocol):
            raise Exception("Must also specify protocol if port range"
                            " is given")
        if parsed_args.sgt:
            if not parsed_args.src_grp:
                raise Exception("Please specify source group with "
                                "--sgt(source group tag) option.")
            body['policy_rule']['tag'] = parsed_args.src_grp
        if parsed_args.dgt:
            if not parsed_args.dst_grp:
                raise Exception("Please specify source group with "
                                "--dgt(destination group tag) option.")
            body['policy_rule']['tag'] = parsed_args.dst_grp
        if parsed_args.tag:
            body['policy_rule']['tag'] = parsed_args.tag
        return body
    except KeyError as err:
        raise Exception("KeyError: " + str(err))


class PolicyRuleCreate(extension.ClientExtensionCreate,
                       PolicyRule):
    """Create an Policy Rule."""

    shell_command = 'policy-rule-create'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name', metavar='<POLICY-RULE-NAME>',
            help=_('Descriptive name for policy rule.'))
        add_known_arguments(self, parser)

    def args2body(self, parsed_args):
        body = args2body(self, parsed_args)
        if parsed_args.tenant_id:
            (body['policy_rule']
                 ['tenant_id']) = parsed_args.tenant_id
        return body


class PolicyRuleList(extension.ClientExtensionList,
                     PolicyRule):
    """List policy rules that belong to a given tenant."""

    shell_command = 'policy-rule-list'
    list_columns = (['id', 'name', 'src_grp', 'dst_grp', 'protocol',
                     'action', 'action_target'])
    pagination_support = True
    sorting_support = True


class PolicyRuleShow(extension.ClientExtensionShow,
                     PolicyRule):
    """Show information of a given policy rule."""

    shell_command = 'policy-rule-show'


class PolicyRuleDelete(extension.ClientExtensionDelete,
                       PolicyRule):
    """Delete a given policy rule."""

    shell_command = 'policy-rule-delete'



# Copyright 2016 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from networking_plumgrid._i18n import _
from neutronclient.common import extension
from neutronclient.common import utils
from oslo_serialization import jsonutils


def _format_ingress_ports(policy_service):
    try:
        return '\n'.join([jsonutils.dumps(ps) for ps in
                          policy_service['ingress_ports']])
    except(TypeError, KeyError):
        return ''


def _format_egress_ports(policy_service):
    try:
        return '\n'.join([jsonutils.dumps(ps) for ps in
                          policy_service['egress_ports']])
    except(TypeError, KeyError):
        return ''


class PolicyService(extension.NeutronClientExtension):
    resource = 'policy_service'
    resource_plural = 'policy_services'
    path = 'policy-services'
    object_path = '/%s' % path
    resource_path = '/%s/%%s' % path
    versions = ['2.0']


def add_known_arguments(self, parser):
    parser.add_argument(
        '--ingress-port',
        metavar='id=ID,name=NAME',
        action='append', dest='ingress_ports',
        type=utils.str2dict,
        help=_('Ingress port to be classified into policy service.'
               ' id=<ingress_port_uuid>,name=<ingress_port_name>'
               '(--ingress-port option can be repeated)'))
    parser.add_argument(
        '--egress-port',
        metavar='id=ID,name=NAME',
        action='append', dest='egress_ports',
        type=utils.str2dict,
        help=_('Egress port to be classified into policy service.'
               ' id=<egress_port_uuid>,name=<egress_port_name>'
               '(--egress-port option can be repeated)'))
    parser.add_argument(
        '--bidirectional-port',
        metavar='id=ID,name=NAME',
        action='append', dest='bidirect_ports',
        type=utils.str2dict,
        help=_('Bidirectional port to be classified into policy service.'
               ' id=<bidirectional_port_uuid>,name=<bidirectional_port_name>'
               '(--bidirectional-port option can be repeated)'))
    parser.add_argument('--description', dest='description',
                        help=_('Description of'
                        ' policy service.'))


def args2body(self, parsed_args):
    try:
        if parsed_args.ingress_ports:
            ingress_ports = parsed_args.ingress_ports
        else:
            ingress_ports = []
        ingress_port_dict = []
        for port in ingress_ports:
            if "id" in port:
                port = {'id': port['id']}
            elif "name" in port:
                port = {'id': port['name']}
            else:
                raise KeyError("ID or Name for ingress port is required.")
            ingress_port_dict.append(port)
        if parsed_args.egress_ports:
            egress_ports = parsed_args.egress_ports
        else:
            egress_ports = []
        egress_port_dict = []
        for port in egress_ports:
            if "id" in port:
                port = {'id': port['id']}
            elif "name" in port:
                port = {'id': port['name']}
            else:
                raise KeyError("ID or Name for egress port is required.")
            egress_port_dict.append(port)
        if parsed_args.bidirect_ports:
            bidirect_ports = parsed_args.bidirect_ports
        else:
            bidirect_ports = []
        bidirect_port_dict = []
        for port in bidirect_ports:
            if "id" in port:
                port = {'id': port['id']}
            elif "name" in port:
                port = {'id': port['name']}
            else:
                raise KeyError("ID or Name for ingress port is required.")
            bidirect_port_dict.append(port)
        if parsed_args.name:
            ps_name = parsed_args.name
            body = {'policy_service': {'name': ps_name}}
        else:
            body = {'policy_service': {}}
        if parsed_args.ingress_ports:
            body['policy_service']['ingress_ports'] = ingress_port_dict
        if parsed_args.egress_ports:
            body['policy_service']['egress_ports'] = egress_port_dict
        if parsed_args.bidirect_ports:
            body['policy_service']['bidirectional_ports'] = bidirect_port_dict
        if parsed_args.description:
            (body['policy_service']
                 ['description']) = parsed_args.description
        return body
    except KeyError as err:
        raise Exception("KeyError: " + str(err))


class PolicyServiceCreate(extension.ClientExtensionCreate,
                          PolicyService):
    """Create a Policy Service."""

    shell_command = 'policy-service-create'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name', metavar='<POLICY-SERVICE-NAME>',
            help=_('Descriptive name for policy service.'))
        add_known_arguments(self, parser)

    def args2body(self, parsed_args):
        body = args2body(self, parsed_args)
        if parsed_args.tenant_id:
            (body['policy_service']
                 ['tenant_id']) = parsed_args.tenant_id
        return body


class PolicyServiceList(extension.ClientExtensionList,
                        PolicyService):
    """List policy services that belong to a given tenant."""

    shell_command = 'policy-service-list'
    list_columns = ['id', 'name', 'ingress_ports', 'egress_ports',
                    'bidirectional_ports']
    pagination_support = True
    sorting_support = True


class PolicyServiceShow(extension.ClientExtensionShow,
                        PolicyService):
    """Show information of a given policy service."""

    shell_command = 'policy-service-show'


class PolicyServiceDelete(extension.ClientExtensionDelete,
                          PolicyService):
    """Delete a given policy service."""

    shell_command = 'policy-service-delete'


class PolicyServiceUpdate(extension.ClientExtensionUpdate,
                          PolicyService):
    """Update a given policy service."""

    shell_command = 'policy-service-update'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--name', metavar='name',
            help=_('Descriptive name for policy service.'))
        parser.add_argument(
            '--add-ingress-port',
            metavar='id=ID,name=NAME',
            action='append', dest='add_ingress_ports',
            type=utils.str2dict,
            help=_('Ingress port to be added to policy service. '
                   'id=<ingress_port_uuid>,name=<ingress_port_name>'
                   '(--add-ingress-port option can be repeated)'))
        parser.add_argument(
            '--add-egress-port',
            metavar='id=ID,name=NAME',
            action='append', dest='add_egress_ports',
            type=utils.str2dict,
            help=_('Egress port to be added to policy service. '
                   'id=<egress_port_uuid>,name=<egress_port_name>'
                   '(--add-egress-port option can be repeated)'))
        parser.add_argument(
            '--remove-ingress-port',
            metavar='id=ID,name=NAME',
            action='append', dest='remove_ingress_ports',
            type=utils.str2dict,
            help=_('Ingress port to be removed from policy service. '
                   'id=<ingress_port_uuid>,name=<ingress_port_name>'
                   '(--remove-ingress_port option can be repeated)'))
        parser.add_argument(
            '--remove-egress-port',
            metavar='id=ID,name=NAME',
            action='append', dest='remove_egress_ports',
            type=utils.str2dict,
            help=_('Egress port to be removed from policy service. '
                   'id=<egress_port_uuid>,name=<egress_port_name>'
                   '(--remove-egress_port option can be repeated)'))
        parser.add_argument(
            '--add-bidirectional-port',
            metavar='id=ID,name=NAME',
            action='append', dest='add_bidirect_ports',
            type=utils.str2dict,
            help=_('Bidirectional port to be added to policy service. '
                  'id=<bidirectional_port_uuid>,name=<bidirectional_port_name>'
                  '(--add-bidirectional-port option can be repeated)'))
        parser.add_argument(
            '--remove-bidirectional-port',
            metavar='id=ID,name=NAME',
            action='append', dest='remove_bidirect_ports',
            type=utils.str2dict,
            help=_('Bidirectional port to be removed from policy service. '
                  'id=<bidirectional_port_uuid>,name=<bidirectional_port_name>'
                  '(--remove-bidirectional-port option can be repeated)'))
        parser.add_argument('--description', dest='description',
                            help=_('Description of'
                            ' policy service.'))

    def args2body(self, parsed_args):
        try:
            if parsed_args.add_ingress_ports:
                add_ingress_ports = parsed_args.add_ingress_ports
            else:
                add_ingress_ports = []
            add_ingress_port_dict = []
            for port in add_ingress_ports:
                if "id" in port:
                    port = {'id': port['id']}
                elif "name" in port:
                    port = {'id': port['name']}
                else:
                    raise KeyError("ID or Name for port is required.")
                add_ingress_port_dict.append(port)
            if parsed_args.add_egress_ports:
                add_egress_ports = parsed_args.add_egress_ports
            else:
                add_egress_ports = []
            add_egress_port_dict = []
            for port in add_egress_ports:
                if "id" in port:
                    port = {'id': port['id']}
                elif "name" in port:
                    port = {'id': port['name']}
                else:
                    raise KeyError("ID or Name for port is required.")
                add_egress_port_dict.append(port)
            if parsed_args.add_bidirect_ports:
                add_bidirect_ports = parsed_args.add_bidirect_ports
            else:
                add_bidirect_ports = []
            add_bidirect_port_dict = []
            for port in add_bidirect_ports:
                if "id" in port:
                    port = {'id': port['id']}
                elif "name" in port:
                    port = {'id': port['name']}
                else:
                    raise KeyError("ID or Name for port is required.")
                add_bidirect_port_dict.append(port)
            if parsed_args.remove_ingress_ports:
                remove_ingress_ports = parsed_args.remove_ingress_ports
            else:
                remove_ingress_ports = []
            remove_ingress_port_dict = []
            for port in remove_ingress_ports:
                if "id" in port:
                    port = {'id': port['id']}
                elif "name" in port:
                    port = {'id': port['name']}
                else:
                    raise KeyError("ID or Name for port is required.")
                remove_ingress_port_dict.append(port)
            if parsed_args.remove_egress_ports:
                remove_egress_ports = parsed_args.remove_egress_ports
            else:
                remove_egress_ports = []
            remove_egress_port_dict = []
            for port in remove_egress_ports:
                if "id" in port:
                    port = {'id': port['id']}
                elif "name" in port:
                    port = {'id': port['name']}
                else:
                    raise KeyError("ID or Name for port is required.")
                remove_egress_port_dict.append(port)
            if parsed_args.remove_bidirect_ports:
                remove_bidirect_ports = parsed_args.remove_bidirect_ports
            else:
                remove_bidirect_ports = []
            remove_bidirect_port_dict = []
            for port in remove_bidirect_ports:
                if "id" in port:
                    port = {'id': port['id']}
                elif "name" in port:
                    port = {'id': port['name']}
                else:
                    raise KeyError("ID or Name for port is required.")
                remove_bidirect_port_dict.append(port)
            if parsed_args.name:
                ps_name = parsed_args.name
                body = {'policy_service': {'name': ps_name}}
            else:
                body = {'policy_service': {}}
            if parsed_args.add_ingress_ports:
                (body['policy_service'][
                 'add_ingress_ports']) = add_ingress_port_dict
            if parsed_args.add_egress_ports:
                (body['policy_service'][
                 'add_egress_ports']) = add_egress_port_dict
            if parsed_args.remove_ingress_ports:
                (body['policy_service']
                 ['remove_ingress_ports']) = remove_ingress_port_dict
            if parsed_args.remove_egress_ports:
                (body['policy_service']
                 ['remove_egress_ports']) = remove_egress_port_dict
            if parsed_args.add_bidirect_ports:
                (body['policy_service']
                 ['add_bidirectional_ports']) = add_bidirect_port_dict
            if parsed_args.remove_bidirect_ports:
                (body['policy_service']
                 ['remove_bidirectional_ports']) = remove_bidirect_port_dict
            if parsed_args.description:
                (body['policy_service']
                 ['description']) = parsed_args.description

            return body
        except KeyError as err:
            raise Exception("KeyError: " + str(err))



# Copyright 2016 PLUMgrid, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import networking_plumgrid
from networking_plumgrid._i18n import _
from neutron.api import extensions
from neutron.api.v2 import attributes as attr
from neutron.api.v2 import base
from neutron.common import exceptions as nexceptions
from neutron import manager
from random import randint

PG_POLICY_RULE = 'policy_rule'
PG_POLICY_RULES = 'policy-rules'

extensions.append_api_extensions_path(
    networking_plumgrid.neutron.plugins.extensions.__path__)


class PolicyRuleInvalidPortRange(nexceptions.InvalidInput):
    message = _("Invalid port range specified '%(port)s'. Following format "
                "is supported e.g. '%(min)s-%(max)s'")


class NoPolicyRuleFound(nexceptions.NotFound):
    message = _("Policy Rule with id '%(id)s' does not exist.")


class NoPolicyGroupFound(nexceptions.NotFound):
    message = _("Policy Group with id '(%(id)s)' specified as "
                "%(policy_config)s does not exist.")


class InvalidPolicyGroupForPolicy(nexceptions.InvalidInput):
    message = _("Policy Group with id '%(id)s' and type '%(type)s' "
                "cannot be specified as '%(policy_config)s' for policy.")


class InvalidPolicyRuleConfig(nexceptions.InvalidInput):
    message = _("Policy Rule only supports source_epg, "
                "destination_epg and service_epg.")


def _validate_port_range(data, valid_values=None):
    if not data:
        return
    try:
        lower_bound, upper_bound = data.split('-')
        lower_bound_val = int(lower_bound)
        upper_bound_val = int(upper_bound)
    except (ValueError, TypeError):
        port_range_min = randint(1, 65535)
        port_range_max = randint(port_range_min, 65535)
        raise PolicyRuleInvalidPortRange(port=data,
                                         min=str(port_range_min),
                                         max=str(port_range_max))

    if (not (lower_bound_val >= 0 and lower_bound_val <= 65535 and
        upper_bound_val >= 0 and upper_bound_val <= 65535) and
        lower_bound_val <= upper_bound_val):
        port_range_min = randint(1, 65535)
        port_range_max = randint(port_range_min, 65535)
        raise PolicyRuleInvalidPortRange(port=data,
                                         min=str(port_range_min),
                                         max=str(port_range_max))


def _validate_action_target(self, action_target):
    #TODO(muawiakhan):FIXME
    return action_target

attr.validators['type:validate_port_range'] = _validate_port_range
attr.validators['type:validate_action_target'] = _validate_action_target
ep_supported_protocols = ['any', 'tcp', 'udp', 'icmp']
ep_supported_actions = ['copy', 'allow']


RESOURCE_ATTRIBUTE_MAP = {
    'policy_rule': {
        'id': {
            'allow_post': False,
            'allow_put': False,
            'validate': {
                'type:uuid': None
            },
            'is_visible': True,
            'primary_key': True
        },
        'name': {
            'allow_post': True,
            'allow_put': False,
            'is_visible': True,
            'default': ''
        },
        'src_grp': {
            'allow_post': True,
            'allow_put': False,
            'is_visible': True,
            'default': None
        },
        'dst_grp': {
            'allow_post': True,
            'allow_put': False,
            'is_visible': True,
            'default': None
        },
        'tag': {
            'allow_post': True,
            'allow_put': False,
            'is_visible': True,
            'default': None
        },
        'protocol': {
            'allow_post': True,
            'allow_put': False,
            'is_visible': True,
            'default': 'any',
            'validate': {
                'type:values': ep_supported_protocols
            }
        },
        'src_port_range': {
            'allow_post': True,
            'allow_put': False,
            'is_visible': True,
            'default': None,
            'validate': {
                'type:validate_port_range': None
            }
        },
        'dst_port_range': {
            'allow_post': True,
            'allow_put': False,
            'is_visible': True,
            'default': None,
            'validate': {
                'type:validate_port_range': None
            }
        },
        'action': {
            'allow_post': True,
            'allow_put': False,
            'is_visible': True,
            'default': None,
            'validate': {'type:values': ep_supported_actions}
        },
        'action_target': {
            'allow_post': True,
            'allow_put': False,
            'is_visible': True,
            'default': None,
            'validate': {
                'type:validate_action_target': None
            },
        },
        'tenant_id': {
            'allow_post': True,
            'allow_put': False,
            'required_by_policy': True,
            'validate': {'type:string': attr.TENANT_ID_MAX_LEN},
            'is_visible': True
        }
    }
}


class Policyrule(extensions.ExtensionDescriptor):

    @classmethod
    def get_name(cls):
        return "Policy Rule"

    @classmethod
    def get_alias(cls):
        return "policy-rule"

    @classmethod
    def get_description(cls):
        return "This API will be used to configure policy rules inside" \
               " policy groups from Neutron and map it in PLUMgrid."

    @classmethod
    def get_namespace(cls):
        return "http://docs.openstack.org/ext/policy_rule" \
               "/api/v2.0"

    @classmethod
    def get_updated(cls):
        return "2016-01-01T00:00:00-00:00"

    @classmethod
    def get_resources(cls):
        exts = []
        plugin = manager.NeutronManager.get_plugin()

        resource_name = PG_POLICY_RULE
        collection_name = PG_POLICY_RULES
        params = RESOURCE_ATTRIBUTE_MAP.get(resource_name, dict())
        controller = base.create_resource(
            collection_name, resource_name, plugin, params)
        ex = extensions.ResourceExtension(collection_name,
                                          controller)
        exts.append(ex)
        return exts



# Copyright 2016 PLUMgrid, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Policy Rule extension unit tests
"""
from networking_plumgrid.neutron.plugins.common import \
    exceptions as p_excep
from networking_plumgrid.neutron.plugins.common import \
    policy_exceptions as policy_excep
from networking_plumgrid.neutron.plugins.extensions import \
    policyrule as ext_pr
from networking_plumgrid.neutron.tests.unit import \
    test_networking_plumgrid as test_pg

from neutron.api import extensions
from neutron import context
from neutron import manager
from neutron.tests.unit.api import test_extensions as test_ext
from oslo_log import log as logging
import uuid

LOG = logging.getLogger(__name__)


class PolicyRuleExtensionManager(object):

    def get_resources(self):
        return ext_pr.Policyrule.get_resources()

    def get_actions(self):
        return []

    def get_request_extensions(self):
        return []


class PolicyRuleTestCase(test_pg.PLUMgridPluginV2TestCase):
    def setUp(self, plugin=None, ext_mgr=None):
        super(PolicyRuleTestCase, self).setUp()
        ext_mgr = PolicyRuleExtensionManager()
        extensions.PluginAwareExtensionManager._instance = None
        self.ext_api = test_ext.setup_extensions_middleware(ext_mgr)


class TestPolicyRule(PolicyRuleTestCase):

    def test_create_policy_rule(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        pr = self._make_pr_dict()

        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_src_grp_epg(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        pr = self._make_pr_dict(src_grp=epg_ret["id"])
        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_dst_grp_epg(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        pr = self._make_pr_dict(dst_grp=epg_ret["id"])
        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_src_grp_epg_invalid(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        pr = self._make_pr_dict(src_grp=str(uuid.uuid4()))
        self.assertRaises(policy_excep.InvalidPolicyRuleConfig,
                          plugin.create_policy_rule, admin_context, pr)

    def test_create_policy_rule_dst_grp_epg_invalid(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        pr = self._make_pr_dict(dst_grp=str(uuid.uuid4()))
        self.assertRaises(policy_excep.InvalidPolicyRuleConfig,
                          plugin.create_policy_rule, admin_context, pr)

    def test_create_policy_rule_src_dst_grp_epg(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        pr = self._make_pr_dict(src_grp=epg_ret["id"], dst_grp=epg_ret["id"])
        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_src_grp_sg(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        sg = self._fake_sg()

        sg_ret = plugin.create_security_group(
                          admin_context, sg)
        pr = self._make_pr_dict(src_grp=sg_ret["id"])
        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_dst_grp_sg(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        sg = self._fake_sg()

        sg_ret = plugin.create_security_group(
                          admin_context, sg)
        pr = self._make_pr_dict(dst_grp=sg_ret["id"])
        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_src_dst_grp_sg(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        sg = self._fake_sg()

        sg_ret = plugin.create_security_group(
                          admin_context, sg)
        pr = self._make_pr_dict(src_grp=sg_ret["id"], dst_grp=sg_ret["id"])
        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_src_epg_dst_sg(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        sg = self._fake_sg()

        sg_ret = plugin.create_security_group(
                          admin_context, sg)
        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        pr = self._make_pr_dict(src_grp=epg_ret["id"], dst_grp=sg_ret["id"])
        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_src_sg_dst_epg(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        sg = self._fake_sg()

        sg_ret = plugin.create_security_group(
                          admin_context, sg)
        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        pr = self._make_pr_dict(src_grp=sg_ret["id"], dst_grp=epg_ret["id"])
        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_action_allow(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        pr = self._make_pr_dict()

        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_action_copy(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        ps = self._make_ps_dict()

        ps_ret = plugin.create_policy_service(admin_context, ps)
        pr = self._make_pr_dict(action="copy", action_target=ps_ret["id"])

        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_action_invalid(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        pr = self._make_pr_dict(action="invalid")
        self.assertRaises(policy_excep.UnsupportedAction,
                          plugin.create_policy_rule, admin_context, pr)

    def test_create_policy_rule_protocol_any(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        pr = self._make_pr_dict(protocol="any")

        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_protocol_tcp(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        pr = self._make_pr_dict(protocol="tcp")

        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_protocol_udp(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        pr = self._make_pr_dict(protocol="udp")

        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_protocol_invalid(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        pr = self._make_pr_dict(protocol="invalid")
        self.assertRaises(policy_excep.UnsupportedProtocol,
                          plugin.create_policy_rule, admin_context, pr)

    def test_create_policy_rule_icmp_with_range(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        pr = self._make_pr_dict(protocol="icmp", src_port_range="10-10")
        self.assertRaises(policy_excep.NotSupportedPortRangeICMP,
                          plugin.create_policy_rule, admin_context, pr)

    def test_create_policy_rule_src_port_range(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        pr = self._make_pr_dict(src_port_range="10-10", protocol="tcp")

        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_dst_port_range(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        pr = self._make_pr_dict(dst_port_range="10-10", protocol="tcp")

        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_invalid_src_port_range(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        pr = self._make_pr_dict(src_port_range="x-y", protocol="tcp")
        self.assertRaises(ext_pr.PolicyRuleInvalidPortRange,
                          plugin.create_policy_rule, admin_context, pr)

    def test_create_policy_rule_invalid_dst_port_range(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        pr = self._make_pr_dict(dst_port_range="x-y", protocol="tcp")
        self.assertRaises(ext_pr.PolicyRuleInvalidPortRange,
                          plugin.create_policy_rule, admin_context, pr)

    def test_create_policy_rule_src_port_range_no_protocol(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        pr = self._make_pr_dict(dst_port_range="10-10")
        self.assertRaises(policy_excep.ProtocolNotSpecified,
                          plugin.create_policy_rule, admin_context, pr)

    def test_create_policy_rule_dst_port_range_no_protocol(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        pr = self._make_pr_dict(dst_port_range="10-10")
        self.assertRaises(policy_excep.ProtocolNotSpecified,
                          plugin.create_policy_rule, admin_context, pr)

    def test_create_policy_rule_action_target(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        ps = self._make_ps_dict()

        ps_ret = plugin.create_policy_service(admin_context, ps)
        pr = self._make_pr_dict(action="copy", action_target=ps_ret["id"])

        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_remote_action_target(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        ps = self._make_ps_dict()

        ps_ret = plugin.create_policy_service(admin_context, ps)
        pr = self._make_pr_dict(action="copy",
                                action_target=ps_ret["tenant_id"] + ":"
                                + ps_ret["name"])
        self.assertRaises(p_excep.PLUMgridException,
                          plugin.create_policy_rule, admin_context, pr)

    def test_create_policy_rule_src_grp_tag(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        ptag = self._make_ptag_dict(tag_type="dot1q",
                                    tag_id="100-200")

        ptag_ret = plugin.create_policy_tag(
                      admin_context, ptag)
        epg = self._make_epg_dict(ptag_id=ptag_ret["id"])

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        pr = self._make_pr_dict(src_grp=epg_ret["id"], tag=ptag_ret["id"])
        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_dst_grp_tag(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        ptag = self._make_ptag_dict(tag_type="dot1q",
                                    tag_id="100-200")

        ptag_ret = plugin.create_policy_tag(
                      admin_context, ptag)
        epg = self._make_epg_dict(ptag_id=ptag_ret["id"])

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        pr = self._make_pr_dict(dst_grp=epg_ret["id"], tag=ptag_ret["id"])
        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_global_tag(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        ptag = self._make_ptag_dict(tag_type="dot1q",
                                    tag_id="100-200")

        ptag_ret = plugin.create_policy_tag(
                      admin_context, ptag)
        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        pr = self._make_pr_dict(dst_grp=epg_ret["id"], tag=ptag_ret["id"])
        pr_ret = plugin.create_policy_rule(admin_context, pr)
        pr_get_ret = plugin.get_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(pr_ret, pr_get_ret)

    def test_create_policy_rule_global_tag_does_not_exist(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        pr = self._make_pr_dict(src_grp=epg_ret["id"], tag=str(uuid.uuid4()))
        self.assertRaises(policy_excep.NoPolicyTagFoundEndpointGroup,
                          plugin.create_policy_rule, admin_context, pr)

    def test_create_policy_rule_action_target_does_not_exist(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        pr = self._make_pr_dict(action="copy", action_target=str(uuid.uuid4()))
        self.assertRaises(policy_excep.NoActionTargetFound,
                          plugin.create_policy_rule, admin_context, pr)

    def test_create_policy_rule_remote_action_target_does_not_exist(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        pr = self._make_pr_dict(action="copy",
                                action_target=str(uuid.uuid4()) + ":" +
                                str(uuid.uuid4()))
        self.assertRaises(p_excep.PLUMgridException,
                          plugin.create_policy_rule, admin_context, pr)

    def test_delete_policy_rule(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        pr = self._make_pr_dict()

        pr_ret = plugin.create_policy_rule(admin_context, pr)
        resp = plugin.delete_policy_rule(admin_context, pr_ret["id"])
        self.assertEqual(None, resp)

    def test_delete_policy_rule_does_not_exist(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        self.assertRaises(policy_excep.NoPolicyRuleFound,
                          plugin.delete_policy_rule, admin_context,
                          str(uuid.uuid4()))

    def test_list_policy_rules(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        pr1 = self._make_pr_dict(name="list_pr1")
        pr_ret1 = plugin.create_policy_rule(admin_context, pr1)
        pr2 = self._make_pr_dict(name="list_pr2")
        pr_ret2 = plugin.create_policy_rule(admin_context, pr2)
        pr3 = self._make_pr_dict(name="list_pr3")
        pr_ret3 = plugin.create_policy_rule(admin_context, pr3)

        pr_list_ret = plugin.get_policy_rules(admin_context)
        self.assertItemsEqual(pr_list_ret, [pr_ret1, pr_ret2, pr_ret3])

    def _make_epg_dict(self, ptag_id=None,
                       name="test_sg_name"):
        epg_dict = {"endpoint_group": {
                   "tenant_id": "test_tenant",
                   "name": name,
                   "description": "test_description",
                   "is_security_group": False}}
        if ptag_id is not None:
            epg_dict["endpoint_group"]["policy_tag_id"] = ptag_id
        return epg_dict

    def _make_epg_update_dict(self, name="test_tenant",
                              description="test_description",
                              add_tag=[],
                              remove_tag=[]):
        return {"endpoint_group": {
                   "tenant_id": "test_tenant",
                   "name": name,
                   "description": description,
                   "add_tag": add_tag,
                   "remove_tag": remove_tag,
                   "is_security_group": False,
                   "policy_tag_id": None}}

    def _make_ps_dict(self, name="test_ps", inports=[], eports=[],
                      bidirect_ports=[]):
        return {"policy_service": {
                    "tenant_id": "test_tenant",
                    "description": "test-ps",
                    "name": name,
                    "ingress_ports": inports,
                    "egress_ports": eports,
                    "bidirectional_ports": bidirect_ports}}

    def _make_pr_dict(self, action_target=None, name="test_policy",
                      src_grp=None, dst_grp=None, action="allow",
                      protocol="any", src_port_range=None,
                      dst_port_range=None, tag=None):
        return {"policy_rule": {
                   "tenant_id": "test_tenant",
                   "name": name,
                   "src_grp": src_grp,
                   "dst_grp": dst_grp,
                   "action": action,
                   "src_port_range": src_port_range,
                   "dst_port_range": dst_port_range,
                   "protocol": protocol,
                   "action_target": action_target,
                   "tag": tag}}

    def _make_ptag_dict(self, tag_type, tag_id=None,
                        floatingip_id=None, floating_ip_address=None,
                        router_id=None):
        return {"policy_tag": {
                               "tenant_id": "test_tenant",
                               "name": "test_name",
                               "tag_type": tag_type,
                               "tag_id": tag_id,
                               "floatingip_id": floatingip_id,
                               "floating_ip_address": floating_ip_address,
                               "router_id": router_id}}

    def _fake_sg(self):
        return {"security_group": {"name": "fake-sg",
                                   "description": "sample-description",
                                   "tenant_id": "test-tenant"}}



# Copyright 2015 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from networking_plumgrid._i18n import _
from neutronclient.common import extension
from neutronclient.neutron import v2_0 as l2gatewayV20


class L2GatewayConnection(extension.NeutronClientExtension):
    resource = 'l2_gateway_connection'
    resource_plural = 'l2_gateway_connections'
    path = 'l2-gateway-connections'
    object_path = '/%s' % path
    resource_path = '/%s/%%s' % path
    versions = ['2.0']


class L2GatewayConnectionCreate(extension.ClientExtensionCreate,
                                L2GatewayConnection):
    """Create l2gateway-connection information."""

    shell_command = 'l2-gateway-connection-create'

    def retrieve_ids(self, client, args):
        gateway_id = l2gatewayV20.find_resourceid_by_name_or_id(
            client, 'l2_gateway', args.gateway_name)
        network_id = l2gatewayV20.find_resourceid_by_name_or_id(
            client, 'network', args.network)
        return (gateway_id, network_id)

    def get_parser(self, parser):
        parser = super(l2gatewayV20.CreateCommand,
                       self).get_parser(parser)
        parser.add_argument(
            'gateway_name', metavar='<GATEWAY-NAME/UUID>',
            help=_('Descriptive name for logical gateway.'))
        parser.add_argument(
            'network', metavar='<NETWORK-NAME/UUID>',
            help=_('Network name or uuid.'))
        parser.add_argument(
            '--segmentation-id',
            dest='seg_id', metavar='<SEGMENTATION-ID>',
            help=_('segmentation-id that will'
                   'be applied to the interfaces of the'
                   'l2 gateway'))
        return parser

    def args2body(self, args):

        neutron_client = self.get_client()
        (gateway_id, network_id) = self.retrieve_ids(neutron_client,
                                                     args)

        body = {'l2_gateway_connection':
                {'l2_gateway_id': gateway_id,
                 'network_id': network_id}}
        if args.seg_id:
            body['l2_gateway_connection']['segmentation_id'] = args.seg_id

        return body


class L2GatewayConnectionList(extension.ClientExtensionList,
                              L2GatewayConnection):
    """List l2gateway-connections."""

    shell_command = 'l2-gateway-connection-list'
    list_columns = ['id', 'l2_gateway_id', 'network_id', 'segmentation_id']
    pagination_support = True
    sorting_support = True


class L2GatewayConnectionShow(extension.ClientExtensionShow,
                              L2GatewayConnection):
    """Show information of a given l2gateway-connection."""

    shell_command = 'l2-gateway-connection-show'
    allow_names = False


class L2GatewayConnectionDelete(extension.ClientExtensionDelete,
                                L2GatewayConnection):
    """Delete a given l2gateway-connection."""

    shell_command = 'l2-gateway-connection-delete'
    allow_names = False



# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""add label sec grp to endpoint

Revision ID: 1b5eb4d4f7d3
Revises: 1ae50f04cdd5, 85d77526e1f6
Create Date: 2016-10-27 01:59:31.893201

"""

# revision identifiers, used by Alembic.
revision = '1b5eb4d4f7d3'
down_revision = ('1ae50f04cdd5', '85d77526e1f6')

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column(
        'pg_endpoints',
        sa.Column('label', sa.String(length=255), nullable=True)
    )

    op.create_table(
        'pg_security_group_endpoint_binding',
        sa.Column('security_group_id', sa.String(length=36), nullable=False),
        sa.Column('endpoint_id', sa.String(length=36), nullable=False),
        sa.ForeignKeyConstraint(['security_group_id'], ['securitygroups.id'],
                                ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['endpoint_id'],
                                ['pg_endpoints.id'])
    )



# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron.tests.api import base
from neutron.tests.tempest import test
from plumgrid_tempest_plugin import config
from plumgrid_tempest_plugin.tests.api import base_l2gw
from random import randint
from tempest_lib.common.utils import data_utils
from tempest_lib import exceptions as lib_exc

CONF = config.CONF


class L2GatewaysNegativeTestJSON(base.BaseNetworkTest):
    _interface = 'json'

    @classmethod
    def resource_setup(cls):
        super(L2GatewaysNegativeTestJSON, cls).resource_setup()
        # Atleast one switch detail should be provided to run the tests
        if (len(CONF.network.l2gw_switch) < 0):
            msg = ('Atleast one switch detail must be defined.')
            raise cls.skipException(msg)
        if not test.is_extension_enabled('l2gateway', 'network'):
            msg = "L2Gateway Extension not enabled."
            raise cls.skipException(msg)

    @test.attr(type=['negative', 'smoke'])
    @test.idempotent_id('b301d83d-3af3-4712-86dc-a6824e9b14e5')
    def test_create_l2gateway_non_admin_user(self):
        gw_name = data_utils.rand_name('l2gw')
        devices = base_l2gw.get_l2gw_body(CONF.network.l2gw_switch)["devices"]
        self.assertRaises(lib_exc.Forbidden,
                          self.client.create_l2_gateway,
                          name=gw_name, devices=devices)

    @test.attr(type=['negative', 'smoke'])
    @test.idempotent_id('68451dfe-b3b5-4eb1-b03f-9935d4a2dbe7')
    def test_list_l2gateway_non_admin_user(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.client.list_l2_gateways)

    @test.attr(type=['negative', 'smoke'])
    @test.idempotent_id('f8589452-7aff-4992-b865-5bb5c41fa755')
    def test_update_l2gateway_non_admin_user(self):
        non_exist_id = data_utils.rand_name('l2gw')
        self.assertRaises(lib_exc.Forbidden,
                          self.client.update_l2_gateway,
                          non_exist_id, name="updated_name")

    @test.attr(type=['negative', 'smoke'])
    @test.idempotent_id('d9f57800-6cae-4770-a2d7-ab60cf7417bf')
    def test_delete_l2gateway_non_admin_user(self):
        non_exist_id = data_utils.rand_name('l2gw')
        self.assertRaises(lib_exc.Forbidden,
                          self.client.delete_l2_gateway,
                          non_exist_id)

    @test.attr(type=['negative', 'smoke'])
    @test.idempotent_id('c6b61a8d-8c82-497d-9fad-9929c9acf035')
    def test_create_l2gateway_connection_non_admin_user(self):
        non_exist_id = data_utils.rand_name('network')
        self.assertRaises(lib_exc.Forbidden,
                          self.client.create_l2_gateway_connection,
                          network_id=non_exist_id, l2_gateway_id=non_exist_id)

    @test.attr(type=['negative', 'smoke'])
    @test.idempotent_id('a56a0180-7d98-414c-9a44-fe47a30fe436')
    def test_list_l2gateway_connection_non_admin_user(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.client.list_l2_gateway_connections)

    @test.attr(type=['negative', 'smoke'])
    @test.idempotent_id('ce42c68d-5c41-4988-8912-233e3fe5c8fd')
    def test_delete_l2gateway_connection_non_admin_user(self):
        non_exist_id = data_utils.rand_name('l2gwconnection')
        self.assertRaises(lib_exc.Forbidden,
                          self.client.delete_l2_gateway_connection,
                          non_exist_id)


class L2GatewaysNegativeAdminTestJSON(base.BaseAdminNetworkTest):
    _interface = 'json'

    @classmethod
    def resource_setup(cls):
        super(L2GatewaysNegativeAdminTestJSON, cls).resource_setup()
        if not test.is_extension_enabled('l2gateway', 'network'):
            msg = "L2Gateway Extension not enabled."
            raise cls.skipException(msg)

    @test.attr(type=['negative', 'smoke'])
    @test.idempotent_id('42067b44-3aff-4428-8305-d0496bd38179')
    def test_delete_l2gw_associated_l2gw_connection(self):
        # Create a network
        name = data_utils.rand_name('network')
        net_body = self.admin_client.create_network(name=name)
        net_id = net_body['network']['id']
        self.addCleanup(self.admin_client.delete_network, net_id)
        # Create an L2Gateway
        gw_name = data_utils.rand_name('l2gw')
        vtep_ifc = "1"
        vtep_ip = "3.3.3.3/32"
        devices = base_l2gw.get_l2gw_body(CONF.network.l2gw_switch)["devices"]
        seg_id = devices[0]['interfaces'][0]['segmentation_id'][0]
        body = self.admin_client.create_l2_gateway(
            name=gw_name, vtep_ifc=vtep_ifc, vtep_ip=vtep_ip, devices=devices)
        l2_gateway = body['l2_gateway']
        self.addCleanup(self.admin_client.delete_l2_gateway, l2_gateway['id'])
        # Create an L2Gateway Connection
        l2_gw_conn_body = self.admin_client.create_l2_gateway_connection(
            l2_gateway_id=l2_gateway['id'], network_id=net_id,
            segmentation_id=seg_id)
        l2_gw_conn_id = l2_gw_conn_body['l2_gateway_connection']['id']
        self.addCleanup(self.admin_client.delete_l2_gateway_connection,
                        l2_gw_conn_id)
        self.assertRaises(lib_exc.Conflict,
                          self.admin_client.delete_l2_gateway,
                          l2_gateway['id'])

    @test.attr(type=['negative', 'smoke'])
    def test_create_l2gw_with_empty_device_name(self):
        # Create an L2Gateway
        seg_id = randint(2, 4094)
        seg_id_str = [str(seg_id)]
        gw_name = data_utils.rand_name('l2gw')
        dev_name = ""
        interface_name = data_utils.rand_name('interface')
        device = [{"device_name": dev_name, "interfaces":
                  [{"name": interface_name, "segmentation_id": seg_id_str}]}]
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_client.create_l2_gateway,
                          name=gw_name, devices=device
                          )

    @test.attr(type=['negative', 'smoke'])
    def test_create_l2gw_with_invalid_vtep_ip(self):
        # Create an L2Gateway
        seg_id = randint(2, 4094)
        seg_id_str = [str(seg_id)]
        gw_name = data_utils.rand_name('l2gw')
        vtep_ifc = "1"
        vtep_ip = "vtep-ip"
        dev_name = "d1"
        interface_name = data_utils.rand_name('interface')
        device = [{"device_name": dev_name, "interfaces":
                  [{"name": interface_name, "segmentation_id": seg_id_str}]}]
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_client.create_l2_gateway,
                          name=gw_name, vtep_ifc=vtep_ifc, vtep_ip=vtep_ip,
                          devices=device
                          )

    @test.attr(type=['negative', 'smoke'])
    def test_create_l2gw_connection_with_invalid_segmentation_id(self):
        # Create an L2Gateway
        gw_name = data_utils.rand_name('l2gw')
        vtep_ifc = "1"
        vtep_ip = "3.3.3.3/32"
        dev_name = data_utils.rand_name('device_name')
        interface_name = data_utils.rand_name('interface')
        devices = [{"device_name": dev_name, "device_ip": "192.168.10.100",
                    "interfaces":
                   [{"segmentation_id": [111], "name": interface_name}]}]
        body = self.admin_client.create_l2_gateway(
            name=gw_name, vtep_ifc=vtep_ifc, vtep_ip=vtep_ip, devices=devices)
        l2_gateway = body['l2_gateway']
        l2_gw_id = l2_gateway['id']
        self.addCleanup(self.admin_client.delete_l2_gateway, l2_gw_id)

        # Create a network
        name = data_utils.rand_name('network')
        net_body = self.admin_client.create_network(name=name)
        net_id = net_body['network']['id']
        self.addCleanup(self.admin_client.delete_network, net_id)
        for i in ['-1', '4095', '4096']:
            seg_id = [i]
            self.assertRaises(lib_exc.BadRequest,
                              self.admin_client.create_l2_gateway_connection,
                              l2_gateway_id=l2_gw_id, network_id=net_id,
                              segmentation_id=seg_id)

    @test.attr(type=['negative', 'smoke'])
    def test_create_l2gw_with_invalid_segmentation_id(self):
        # Create an L2Gateway
        gw_name = data_utils.rand_name('l2gw')
        dev_name = data_utils.rand_name('device_name')
        interface_name = data_utils.rand_name('interface')
        for i in ['-1', '4095', '4096']:
            seg_id = [i]
            device = [{"device_name": dev_name, "device_ip": "192.168.10.100",
                       "interfaces":
                      [{"name": interface_name, "segmentation_id": seg_id}]}]
            self.assertRaises(lib_exc.BadRequest,
                              self.admin_client.create_l2_gateway,
                              name=gw_name, devices=device
                              )

    @test.attr(type=['negative', 'smoke'])
    def test_create_l2gw_with_empty_interface_name(self):
        # Create an L2Gateway
        seg_id = randint(2, 4094)
        seg_id_str = [str(seg_id)]
        gw_name = data_utils.rand_name('l2gw')
        dev_name = data_utils.rand_name('device')
        interface_name = ""
        device = [{"device_name": dev_name, "device_ip": "192.168.10.100",
                   "interfaces":
                  [{"name": interface_name, "segmentation_id": seg_id_str}]}]
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_client.create_l2_gateway,
                          name=gw_name, devices=device
                          )

    @test.attr(type=['negative', 'smoke'])
    def test_delete_non_existent_l2gateway(self):
        non_exist_id = data_utils.rand_name('l2gw')
        self.assertRaises(lib_exc.NotFound,
                          self.admin_client.delete_l2_gateway,
                          non_exist_id)

    @test.attr(type=['negative', 'smoke'])
    def test_delete_non_existent_l2gateway_connection(self):
        non_exist_id = data_utils.rand_name('l2gwConnection')
        self.assertRaises(lib_exc.NotFound,
                          self.admin_client.delete_l2_gateway_connection,
                          non_exist_id)

    @test.attr(type=['negative', 'smoke'])
    def test_create_l2gw_connection_with_invalid_network_name(self):
        # Create an L2Gateway
        gw_name = data_utils.rand_name('l2gw')
        vtep_ifc = "1"
        vtep_ip = "3.3.3.3/32"
        devices = base_l2gw.get_l2gw_body(CONF.network.l2gw_switch)["devices"]
        seg_id = devices[0]['interfaces'][0]['segmentation_id'][0]
        body = self.admin_client.create_l2_gateway(
            name=gw_name, vtep_ifc=vtep_ifc, vtep_ip=vtep_ip, devices=devices)
        l2_gateway = body['l2_gateway']
        l2_gw_id = l2_gateway['id']
        self.addCleanup(self.admin_client.delete_l2_gateway, l2_gw_id)

        # Create a network
        net_id = "network"
        self.assertRaises(lib_exc.ServerFault,
                          self.admin_client.create_l2_gateway_connection,
                          l2_gateway_id=l2_gw_id, network_id=net_id,
                          segmentation_id=seg_id
                          )

    @test.attr(type=['negative', 'smoke'])
    def test_update_gateway_with_invalid_device_name(self):
        # Create an L2Gateway
        gw_name = data_utils.rand_name('l2gw')
        vtep_ifc = "1"
        vtep_ip = "3.3.3.3/32"
        devices = base_l2gw.get_l2gw_body(CONF.network.l2gw_switch)["devices"]
        body = self.admin_client.create_l2_gateway(
            name=gw_name, vtep_ifc=vtep_ifc, vtep_ip=vtep_ip, devices=devices)
        l2_gateway = body['l2_gateway']
        self.addCleanup(self.admin_client.delete_l2_gateway, l2_gateway['id'])
        device_1 = [{"device_name": ""}]

        # Create a connection again for same L2Gateway and Network
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_client.update_l2_gateway,
                          gw_name, devices=device_1
                          )

    @test.attr(type=['negative', 'smoke'])
    def test_create_l2gw_and_l2gw_connection_both_without_seg_id(self):
        # Create an L2Gateway
        gw_name = data_utils.rand_name('l2gw')
        vtep_ifc = "1"
        vtep_ip = "3.3.3.3/32"
        devices = base_l2gw.get_l2gw_body(CONF.network.l2gw_switch)["devices"]
        body = self.admin_client.create_l2_gateway(
            name=gw_name, vtep_ifc=vtep_ifc, vtep_ip=vtep_ip, devices=devices)
        l2_gateway = body['l2_gateway']
        l2_gw_id = l2_gateway['id']
        self.addCleanup(self.admin_client.delete_l2_gateway, l2_gw_id)

        # Create a network
        name = data_utils.rand_name('network')
        net_body = self.admin_client.create_network(name=name)
        net_id = net_body['network']['id']
        self.addCleanup(self.admin_client.delete_network, net_id)
        self.assertRaises(lib_exc.Conflict,
                          self.admin_client.create_l2_gateway_connection,
                          l2_gateway_id=l2_gw_id, network_id=net_id
                          )



# Copyright 2016 PLUMgrid, Inc. All Rights Reserved.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import mock
from networking_plumgrid.neutron.tests.unit.cli import test_cli20
from networking_plumgrid.neutronclient.policy import (
    policy_rule as pr)
from neutronclient import shell
import sys


class CLITestV20ExtensionPolicyRuleJSON(test_cli20.CLITestV20Base):
    def setUp(self):
        # need to mock before super because extensions loaded on instantiation
        self._mock_extension_loading()
        super(CLITestV20ExtensionPolicyRuleJSON,
              self).setUp(plurals={'tags': 'tag'})

    def _mock_extension_loading(self):
        ext_pkg = 'neutronclient.common.extension'
        contrib = mock.patch(ext_pkg + '._discover_via_entry_points').start()
        contrib.return_value = [("policy_rule", pr)]
        return contrib

    def test_ext_cmd_loaded(self):
        """Tests policy rule commands loaded."""
        shell.NeutronShell('2.0')
        ext_cmd = {'policy-rule-list':
                   pr.PolicyRuleList,
                   'policy-rule-create':
                   pr.PolicyRuleCreate,
                   'policy-rule-delete':
                   pr.PolicyRuleDelete,
                   'policy-rule-show':
                   pr.PolicyRuleShow}
        self.assertDictContainsSubset(ext_cmd, shell.COMMANDS['2.0'])

    def test_create_policy_rule_name(self):
        # Create policy rule: pr1.
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        args = [name]
        position_names = ['name']
        position_values = [name]
        self._test_create_resource(resource, cmd, name, 'myid', args,
                                   position_names, position_values)

    def test_create_policy_rule_src_grp(self):
        # Create policy rule: pr1.
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        src_grp = 'epg1'
        args = [name, '--src-grp', src_grp]
        position_names = ['name', 'src_grp']
        position_values = [name, src_grp]
        self._test_create_resource(resource, cmd, name, 'myid', args,
                                   position_names, position_values)

    def test_create_policy_rule_dst_grp(self):
        # Create policy rule: pr1.
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        dst_grp = 'epg1'
        args = [name, '--dst-grp', dst_grp]
        position_names = ['name', 'dst_grp']
        position_values = [name, dst_grp]
        self._test_create_resource(resource, cmd, name, 'myid', args,
                                   position_names, position_values)

    def test_create_policy_rule_proto_any(self):
        # Create policy rule: pr1.
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        proto = 'any'
        args = [name, '--protocol', proto]
        position_names = ['name', 'protocol']
        position_values = [name, proto]
        self._test_create_resource(resource, cmd, name, 'myid', args,
                                   position_names, position_values)

    def test_create_policy_rule_proto_tcp(self):
        # Create policy rule: pr1.
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        proto = 'tcp'
        args = [name, '--protocol', proto]
        position_names = ['name', 'protocol']
        position_values = [name, proto]
        self._test_create_resource(resource, cmd, name, 'myid', args,
                                   position_names, position_values)

    def test_create_policy_rule_proto_udp(self):
        # Create policy rule: pr1.
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        proto = 'udp'
        args = [name, '--protocol', proto]
        position_names = ['name', 'protocol']
        position_values = [name, proto]
        self._test_create_resource(resource, cmd, name, 'myid', args,
                                   position_names, position_values)

    def test_create_policy_rule_proto_invalid(self):
        # Create policy rule: pr1.
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        proto = 'invalid'
        args = [name, '--protocol', proto]
        position_names = ['name', 'protocol']
        position_values = [name, proto]
        self.assertRaises(Exception, self._test_create_resource, resource,
                          cmd, name, 'myid', args,
                          position_names, position_values)

    def test_create_policy_rule_action_allow(self):
        # Create policy rule: pr1.
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        action = 'allow'
        args = [name, '--action', action]
        position_names = ['name', 'action']
        position_values = [name, action]
        self._test_create_resource(resource, cmd, name, 'myid', args,
                                   position_names, position_values)

    def test_create_policy_rule_action_copy(self):
        # Create policy rule: pr1.
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        action = 'copy'
        args = [name, '--action', action]
        position_names = ['name', 'action']
        position_values = [name, action]
        self.assertRaises(Exception, self._test_create_resource, resource,
                          cmd, name, 'myid', args,
                          position_names, position_values)

    def test_create_policy_rule_action_invalid(self):
        # Create policy rule: pr1.
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        action = 'invalid'
        args = [name, '--action', action]
        position_names = ['name', 'action']
        position_values = [name, action]
        self.assertRaises(Exception, self._test_create_resource, resource,
                          cmd, name, 'myid', args,
                          position_names, position_values)

    def test_create_policy_rule_action_target(self):
        # Create policy rule: pr1.
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        action_target = 'ps1'
        args = [name, '--action-target', action_target]
        position_names = ['name', 'action_target']
        position_values = [name, action_target]
        self.assertRaises(Exception, self._test_create_resource, resource,
                          cmd, name, 'myid', args,
                          position_names, position_values)

    def test_create_policy_rule_remote_action_target(self):
        # Create policy rule: pr1.
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        action_target = 'tenant:ps1'
        args = [name, '--action-target', action_target]
        position_names = ['name', 'action_target']
        position_values = [name, action_target]
        self.assertRaises(Exception, self._test_create_resource, resource,
                          cmd, name, 'myid', args,
                          position_names, position_values)

    def test_create_policy_rule_copy_action_target(self):
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        action_target = 'ps1'
        action = 'copy'
        args = [name, '--action-target', action_target, '--action', action]
        position_names = ['name', 'action_target', 'action']
        position_values = [name, action_target, action]
        self._test_create_resource(resource, cmd, name, 'myid', args,
                                   position_names, position_values)

    def test_create_policy_rule_copy_remote_action_target(self):
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        action_target = 'tenant:ps1'
        action = 'copy'
        args = [name, '--action-target', action_target, '--action', action]
        position_names = ['name', 'action_target', 'action']
        position_values = [name, action_target, action]
        self._test_create_resource(resource, cmd, name, 'myid', args,
                                   position_names, position_values)

    def test_create_policy_rule_copy_invalid_remote_action_target(self):
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        action_target = 'tenant:ps1:invalid'
        action = 'copy'
        args = [name, '--action-target', action_target, '--action', action]
        position_names = ['name', 'action_target', 'action']
        position_values = [name, action_target, action]
        self.assertRaises(Exception, self._test_create_resource, resource,
                          cmd, name, 'myid', args,
                          position_names, position_values)

    def test_create_policy_rule_allow_action_target(self):
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        action_target = 'ps1'
        action = 'allow'
        args = [name, '--action-target', action_target, '--action', action]
        position_names = ['name', 'action_target', 'action']
        position_values = [name, action_target, action]
        self.assertRaises(Exception, self._test_create_resource, resource,
                          cmd, name, 'myid', args,
                          position_names, position_values)

    def test_create_policy_rule_sgt(self):
        # Create policy rule: pr1.
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        args = [name, '--sgt']
        position_names = ['name']
        position_values = [name]
        self.assertRaises(Exception, self._test_create_resource, resource,
                          cmd, name, 'myid', args,
                          position_names, position_values)

    def test_create_policy_rule_dgt(self):
        # Create policy rule: pr1.
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        args = [name, '--dgt']
        position_names = ['name']
        position_values = [name]
        self.assertRaises(Exception, self._test_create_resource, resource,
                          cmd, name, 'myid', args,
                          position_names, position_values)

    def test_create_policy_rule_tag(self):
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        tag = 'ptag1'
        args = [name, '--tag', tag]
        position_names = ['name', 'tag']
        position_values = [name, tag]
        self._test_create_resource(resource, cmd, name, 'myid', args,
                                   position_names, position_values)

    def test_create_policy_rule_src_port_range(self):
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        src_port_range = '1-65535'
        args = [name, '--src-port-range', src_port_range]
        position_names = ['name', 'src_port_range']
        position_values = [name, src_port_range]
        self.assertRaises(Exception, self._test_create_resource, resource,
                          cmd, name, 'myid', args,
                          position_names, position_values)

    def test_create_policy_rule_dst_port_range(self):
        resource = 'policy_rule'
        cmd = pr.PolicyRuleCreate(test_cli20.MyApp(
                                  sys.stdout), None)
        name = 'pr1'
        dst_port_range = '1-65535'
        args = [name, '--dest-port-range', dst_port_range]
        position_names = ['name', 'dst_port_range']
        position_values = [name, dst_port_range]
        self.assertRaises(Exception, self._test_create_resource, resource,
                          cmd, name, 'myid', args,
                          position_names, position_values)

    def test_list_policy_rules(self):
        # list policy rules: -D.
        resources = "policy_rules"
        cmd = pr.PolicyRuleList(test_cli20.MyApp(
                                sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_list_policy_rules_sort(self):
        # list policy rules:
        # --sort-key name --sort-key id --sort-key asc --sort-key desc
        resources = "policy_rules"
        cmd = pr.PolicyRuleList(test_cli20.MyApp(
                                sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_policy_rules_limit(self):
        # list policy rules: -P.
        resources = "policy_rules"
        cmd = pr.PolicyRuleList(test_cli20.MyApp(
                                sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_delete_policy_rule(self):
        # Delete policy rule: myid.
        resource = "policy_rule"
        cmd = pr.PolicyRuleDelete(test_cli20.MyApp(
                                  sys.stdout), None)
        my_id = 'myid'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)

    def test_show_policy_rule(self):
        # Show policy rule: --fields id.
        resource = 'policy_rule'
        cmd = pr.PolicyRuleShow(test_cli20.MyApp(
                                sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args,
                                 ['id', 'name'])



# Copyright 2016 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from networking_plumgrid._i18n import _
from neutronclient.common import extension


class PolicyTag(extension.NeutronClientExtension):
    resource = 'policy_tag'
    resource_plural = 'policy_tags'
    path = 'policy-tags'
    object_path = '/%s' % path
    resource_path = '/%s/%%s' % path
    versions = ['2.0']


def args2body(self, parsed_args):
    try:
        if parsed_args.name:
            ptag_name = parsed_args.name
            body = {'policy_tag': {'name': ptag_name}}
        else:
            body = {'policy_tag': {}}
        if parsed_args.tag_type:
            if (str(parsed_args.tag_type).lower() == 'fip' or
                str(parsed_args.tag_type).lower() == 'dot1q' or
                str(parsed_args.tag_type).lower() == 'nsh'):
                body['policy_tag']['tag_type'] \
                    = parsed_args.tag_type
            else:
                raise Exception("Supported values for policy tag type are:"
                                " 'fip', 'dot1q', 'nsh'")
        else:
            raise Exception("Policy tag type is required to be specified. "
                            "Supported values for policy tag type are:"
                            " 'fip', 'dot1q', 'nsh'")
        if parsed_args.tag_id:
            body['policy_tag']['tag_id'] = parsed_args.tag_id
        if parsed_args.router_id:
            body['policy_tag']['router_id'] = parsed_args.router_id
        if parsed_args.floatingip_id:
            body['policy_tag']['floatingip_id'] = parsed_args.floatingip_id
        if (parsed_args.tag_type and parsed_args.tag_type.lower() == 'fip'
            and not parsed_args.floatingip_id):
            raise Exception("Floating IP UUID must be specified when "
                            "using tag type=fip")
        if (parsed_args.tag_type and (parsed_args.tag_type.lower() == 'dot1q'
            or parsed_args.tag_type.lower() == 'nsh')
            and not parsed_args.tag_id):
            raise Exception("ID in range (257-2047) must be specified when "
                            "using tag type=dot1q or type=nsh")
        if (parsed_args.router_id and parsed_args.tag_type.lower() != 'fip'):
            raise Exception("Tag type='fip' must be specified when using "
                            "Router ID")
        if (parsed_args.tag_type.lower() == 'fip' and parsed_args.tag_id):
            raise Exception("Tag type=='fip' does not support tag id.")
        if (parsed_args.floatingip_id and
            parsed_args.tag_type.lower() != 'fip'):
            raise Exception('Floating ip cannot be associated with tag type:'
                            + parsed_args.tag_type.lower())
        return body
    except KeyError as err:
        raise Exception("KeyError: " + str(err))


class PolicyTagCreate(extension.ClientExtensionCreate,
                      PolicyTag):
    """Create a Policy Tag."""

    shell_command = 'policy-tag-create'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name', metavar='<POLICY-TAG-NAME>',
            help=_('Descriptive name for policy tag.'))
        parser.add_argument('--type', dest='tag_type',
                            help=_('Type'
                            ' of policy tag. Options:'
                            ' fip, dot1q, nsh'))
        parser.add_argument('--floating-ip', dest='floatingip_id',
                        help=_('UUID of Floating IP to associate '
                               ' with the Policy Tag.'))
        parser.add_argument('--tag-id', dest='tag_id',
                        help=_('ID in range 257-2047 '))
        parser.add_argument('--router-id', dest='router_id',
                        help=_('Router ID to be specified in case '
                               'of multiple External Gateways, when '
                               'associating a Floating IP.'))

    def args2body(self, parsed_args):
        body = args2body(self, parsed_args)
        if parsed_args.tenant_id:
            (body['policy_tag']
                 ['tenant_id']) = parsed_args.tenant_id
        return body


class PolicyTagList(extension.ClientExtensionList,
                    PolicyTag):
    """List policy tags that belong to a given tenant."""

    shell_command = 'policy-tag-list'
    list_columns = ['id', 'name', 'tag_type', 'tag_id', 'floating_ip_address']
    pagination_support = True
    sorting_support = True


class PolicyTagShow(extension.ClientExtensionShow,
                    PolicyTag):
    """Show information of a given policy tag."""

    shell_command = 'policy-tag-show'


class PolicyTagDelete(extension.ClientExtensionDelete,
                      PolicyTag):
    """Delete a given policy tag."""

    shell_command = 'policy-tag-delete'


class PolicyTagUpdate(extension.ClientExtensionUpdate,
                      PolicyTag):
    """Update a given policy-tag."""

    shell_command = 'policy-tag-update'



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

'''
Interface for database access.

Usage:

The underlying driver is loaded . SQLAlchemy is currently the only
supported backend.
'''

from oslo_config import cfg
from oslo_db import api

CONF = cfg.CONF


_BACKEND_MAPPING = {'sqlalchemy':
                    'networking_plumgrid.neutron.plugins.db.sqlal.api'}

IMPL = api.DBAPI.from_config(CONF, backend_mapping=_BACKEND_MAPPING)


def get_engine():
    return IMPL.get_engine()


def get_session():
    return IMPL.get_session()


def pg_lock_create(uuid):
    return IMPL.pg_lock_create(uuid)


def pg_lock_get(uuid):
    return IMPL.pg_lock_get(uuid)


def pg_lock_get_id(uuid):
    return IMPL.pg_lock_get_id(uuid)


def pg_lock_steal(uuid):
    return IMPL.pg_lock_steal(uuid)


def pg_lock_release(uuid):
    return IMPL.pg_lock_release(uuid)



# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""l2_gateway

Revision ID: 1be40493866d
Revises: 2df53910135e
Create Date: 2015-12-08 02:54:29.072922

"""

# revision identifiers, used by Alembic.
revision = '1be40493866d'
down_revision = '2df53910135e'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table(
        'pgl2gateways',
        sa.Column('id', sa.String(length=36), nullable=False),
        sa.Column('name', sa.String(length=255), nullable=True),
        sa.Column('tenant_id', sa.String(length=36), nullable=True),
        sa.Column('vtep_ifc', sa.String(length=36), nullable=True),
        sa.Column('vtep_ip', sa.String(length=36), nullable=True),
        sa.PrimaryKeyConstraint('id')
    )

    op.create_table(
        'pgl2gatewaydevices',
        sa.Column('id', sa.String(length=36), nullable=False),
        sa.Column('device_name', sa.String(length=255), nullable=False),
        sa.Column('device_ip', sa.String(length=36), nullable=True),
        sa.Column('l2_gateway_id', sa.String(length=36), nullable=False),
        sa.ForeignKeyConstraint(
            ['l2_gateway_id'], ['pgl2gateways.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )

    op.create_table(
        'pgl2gatewayinterfaces',
        sa.Column('id', sa.String(length=36), nullable=False),
        sa.Column('interface_name', sa.String(length=255), nullable=False),
        sa.Column('device_id', sa.String(length=36), nullable=False),
        sa.Column('segmentation_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(
            ['device_id'], ['pgl2gatewaydevices.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )

    op.create_table(
        'pgl2gatewayconnections',
        sa.Column('id', sa.String(length=36), nullable=False),
        sa.Column('tenant_id', sa.String(length=36), nullable=True),
        sa.Column('l2_gateway_id', sa.String(length=36), nullable=True),
        sa.Column('network_id', sa.String(length=36), nullable=True),
        sa.Column('port_id', sa.String(length=36), nullable=True),
        sa.Column('segmentation_id', sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ['l2_gateway_id'], ['pgl2gateways.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(
            ['network_id'], ['networks.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )



# Copyright 2016 PLUMgrid, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Endpoint extension unit tests
"""
from networking_plumgrid.neutron.plugins.common import \
    exceptions as p_excep
from networking_plumgrid.neutron.plugins.common import \
    policy_exceptions as policy_excep
from networking_plumgrid.neutron.plugins.extensions import \
    endpoint as ext_ep
from networking_plumgrid.neutron.tests.unit import \
    test_networking_plumgrid as test_pg

from neutron.api import extensions
from neutron.api.v2 import attributes
from neutron import context
from neutron import manager
from neutron.tests.unit.api import test_extensions as test_ext
from oslo_log import log as logging
import uuid

LOG = logging.getLogger(__name__)


class EndpointExtensionManager(object):

    def get_resources(self):
        return ext_ep.Endpoint.get_resources()

    def get_actions(self):
        return []

    def get_request_extensions(self):
        return []


class EndpointTestCase(test_pg.PLUMgridPluginV2TestCase):
    def setUp(self, plugin=None, ext_mgr=None):
        super(EndpointTestCase, self).setUp()
        ext_mgr = EndpointExtensionManager()
        extensions.PluginAwareExtensionManager._instance = None
        self.ext_api = test_ext.setup_extensions_middleware(ext_mgr)


class TestEndpoint(EndpointTestCase):
    def test_create_endpoint_with_ip_mask(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep = self._make_ep_dict(ip_mask='0.0.0.0/0',
                                ep_groups=[{'id': epg_ret['id']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        ep["endpoint"]["id"] = ep_ret["id"]
        self.assertEqual(ep_ret, ep["endpoint"])

    def test_create_endpoint_with_ip_port_mask(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep = self._make_ep_dict(ip_port='1.1.1.1_100_8',
                                ep_groups=[{'id': epg_ret['id']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        ep["endpoint"]["id"] = ep_ret["id"]
        self.assertEqual(ep_ret, ep["endpoint"])

    def test_create_endpoint_with_port(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        tenant_context = context.Context('', 'not_admin')

        network = self._fake_network()
        network["network"]["tenant_id"] = tenant_context.tenant_id
        network_ret = plugin.create_network(tenant_context, network)

        port = self._fake_port(network_ret["id"])
        port_ret = plugin.create_port(
                      admin_context, port)

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep = self._make_ep_dict(port=port_ret["id"],
                                ep_groups=[{'id': epg_ret['id']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        ep["endpoint"]["id"] = ep_ret["id"]
        self.assertEqual(ep_ret, ep["endpoint"])

    def test_create_endpoint_with_ip_mask_epg_name(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep = self._make_ep_dict(ip_mask='0.0.0.0/0',
                                ep_groups=[{'id': epg_ret['name']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        ep["endpoint"]["id"] = ep_ret["id"]
        self.assertEqual(ep_ret, ep["endpoint"])

    def test_create_endpoint_with_ip_port_mask_epg_name(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep = self._make_ep_dict(ip_port='1.1.1.1_100_8',
                                ep_groups=[{'id': epg_ret['name']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        ep["endpoint"]["id"] = ep_ret["id"]
        self.assertEqual(ep_ret, ep["endpoint"])

    def test_create_endpoint_with_port_epg_name(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        tenant_context = context.Context('', 'not_admin')

        network = self._fake_network()
        network["network"]["tenant_id"] = tenant_context.tenant_id
        network_ret = plugin.create_network(tenant_context, network)

        port = self._fake_port(network_ret["id"])
        port_ret = plugin.create_port(
                      admin_context, port)

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep = self._make_ep_dict(port=port_ret["id"],
                                ep_groups=[{'id': epg_ret['name']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        ep["endpoint"]["id"] = ep_ret["id"]
        self.assertEqual(ep_ret, ep["endpoint"])

    def test_create_endpoint_with_port_and_epg_name(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        tenant_context = context.Context('', 'not_admin')

        network = self._fake_network()
        network["network"]["tenant_id"] = tenant_context.tenant_id
        network_ret = plugin.create_network(tenant_context, network)

        port = self._fake_port(network_ret["id"])
        port_ret = plugin.create_port(
                      admin_context, port)

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep = self._make_ep_dict(port=port_ret["name"],
                                ep_groups=[{'id': epg_ret['name']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        ep["endpoint"]["id"] = ep_ret["id"]
        self.assertEqual(ep_ret, ep["endpoint"])

    def test_create_endpoint_with_ip_mask_and_ip_port(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep = self._make_ep_dict(ip_mask='0.0.0.0/0',
                                ip_port="1.1.1.1_100_8",
                                ep_groups=[{'id': epg_ret['id']}])
        self.assertRaises(policy_excep.MultipleAssociationForEndpoint,
                          plugin.create_endpoint, admin_context, ep)

    def test_create_endpoint_with_port_and_ip_mask(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        tenant_context = context.Context('', 'not_admin')

        network = self._fake_network()
        network["network"]["tenant_id"] = tenant_context.tenant_id
        network_ret = plugin.create_network(tenant_context, network)

        port = self._fake_port(network_ret["id"])
        port_ret = plugin.create_port(
                      admin_context, port)

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep = self._make_ep_dict(port=port_ret["id"],
                                ip_mask='0.0.0.0/0',
                                ep_groups=[{'id': epg_ret['id']}])
        self.assertRaises(policy_excep.MultipleAssociationForEndpoint,
                          plugin.create_endpoint, admin_context, ep)

    def test_create_endpoint_with_port_and_ip_port(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        tenant_context = context.Context('', 'not_admin')

        network = self._fake_network()
        network["network"]["tenant_id"] = tenant_context.tenant_id
        network_ret = plugin.create_network(tenant_context, network)

        port = self._fake_port(network_ret["id"])
        port_ret = plugin.create_port(
                      admin_context, port)

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep = self._make_ep_dict(port=port_ret["id"],
                                ip_port='1.1.1.1_100_8',
                                ep_groups=[{'id': epg_ret['id']}])
        self.assertRaises(policy_excep.MultipleAssociationForEndpoint,
                          plugin.create_endpoint, admin_context, ep)

    def test_create_endpoint_with_duplicate_epg(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        tenant_context = context.Context('', 'not_admin')

        network = self._fake_network()
        network["network"]["tenant_id"] = tenant_context.tenant_id
        network_ret = plugin.create_network(tenant_context, network)

        port = self._fake_port(network_ret["id"])
        port_ret = plugin.create_port(
                      admin_context, port)

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep = self._make_ep_dict(port=port_ret["id"],
                                ep_groups=[{'id': epg_ret['id']},
                                {'id': epg_ret['id']}])
        self.assertRaises(policy_excep.DuplicateEndpointGroup,
                          plugin.create_endpoint, admin_context, ep)

    def test_create_endpoint_with_security_group(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        sg = self._fake_sg()

        sg_ret = plugin.create_security_group(admin_context, sg)
        ep = self._make_ep_dict(ip_mask='0.0.0.0/0',
                                ep_groups=[{'id': sg_ret['id']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        ep["endpoint"]["id"] = ep_ret["id"]
        self.assertEqual(ep_ret, ep["endpoint"])

    def test_create_endpoint_non_existent_port(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(admin_context, epg)
        ep = self._make_ep_dict(port='fake-port',
                                ep_groups=[{'id': epg_ret['id']}])
        self.assertRaises(p_excep.PLUMgridException,
                          plugin.create_endpoint, admin_context, ep)

    def test_create_endpoint_port_owner_not_nova(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        tenant_context = context.Context('', 'not_admin')

        network = self._fake_network()
        network["network"]["tenant_id"] = tenant_context.tenant_id
        network_ret = plugin.create_network(tenant_context, network)

        port = self._fake_port(network_ret["id"], device_owner="not:nova")
        port_ret = plugin.create_port(
                      admin_context, port)

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep = self._make_ep_dict(port=port_ret["id"],
                                ep_groups=[{'id': epg_ret['id']}])
        self.assertRaises(policy_excep.InvalidPortForEndpoint,
                          plugin.create_endpoint, admin_context, ep)

    def test_create_endpoint_port_in_use(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        tenant_context = context.Context('', 'not_admin')

        network = self._fake_network()
        network["network"]["tenant_id"] = tenant_context.tenant_id
        network_ret = plugin.create_network(tenant_context, network)

        port = self._fake_port(network_ret["id"])
        port_ret = plugin.create_port(
                      admin_context, port)

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep1 = self._make_ep_dict(port=port_ret["id"],
                                 ep_groups=[{'id': epg_ret['id']}])
        plugin.create_endpoint(admin_context, ep1)
        ep2 = self._make_ep_dict(name="test_ep2",
                                 port=port_ret["id"],
                                 ep_groups=[{'id': epg_ret['id']}])
        self.assertRaises(ext_ep.PortInUse,
                          plugin.create_endpoint, admin_context, ep2)

    def test_create_endpoint_invalid_epg(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        tenant_context = context.Context('', 'not_admin')

        network = self._fake_network()
        network["network"]["tenant_id"] = tenant_context.tenant_id
        network_ret = plugin.create_network(tenant_context, network)

        port = self._fake_port(network_ret["id"])
        port_ret = plugin.create_port(
                      admin_context, port)

        ep = self._make_ep_dict(port=port_ret["id"],
                                ep_groups=[{'id': str(uuid.uuid4())}])
        self.assertRaises(ext_ep.NoEndpointGroupFound,
                          plugin.create_endpoint, admin_context, ep)

    def test_delete_endpoint_ip_mask(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep = self._make_ep_dict(ip_mask='0.0.0.0/0',
                                ep_groups=[{'id': epg_ret['id']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        resp = plugin.delete_endpoint(admin_context, ep_ret["id"])
        self.assertEqual(None, resp)

    def test_delete_endpoint_ip_port(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep = self._make_ep_dict(ip_port='1.1.1.1_100_8',
                                ep_groups=[{'id': epg_ret['id']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        resp = plugin.delete_endpoint(admin_context, ep_ret["id"])
        self.assertEqual(None, resp)

    def test_delete_endpoint_port(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        tenant_context = context.Context('', 'not_admin')

        network = self._fake_network()
        network["network"]["tenant_id"] = tenant_context.tenant_id
        network_ret = plugin.create_network(tenant_context, network)

        port = self._fake_port(network_ret["id"])
        port_ret = plugin.create_port(
                      admin_context, port)

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep = self._make_ep_dict(port=port_ret["id"],
                                ep_groups=[{'id': epg_ret['id']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        resp = plugin.delete_endpoint(admin_context, ep_ret["id"])
        self.assertEqual(None, resp)

    def test_delete_endpoint_does_not_exist(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        self.assertRaises(ext_ep.NoEndpointFound,
                          plugin.delete_endpoint, admin_context,
                          str(uuid.uuid4()))

    def test_list_endpoints(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep1 = self._make_ep_dict(ip_mask='0.0.0.0/0',
                                 ep_groups=[{'id': epg_ret['id']}])
        ep_ret1 = plugin.create_endpoint(admin_context, ep1)
        ep2 = self._make_ep_dict(ip_mask='1.0.0.0/0',
                                 ep_groups=[{'id': epg_ret['id']}])
        ep_ret2 = plugin.create_endpoint(admin_context, ep2)
        ep3 = self._make_ep_dict(ip_mask='2.0.0.0/0',
                                 ep_groups=[{'id': epg_ret['id']}])
        ep_ret3 = plugin.create_endpoint(admin_context, ep3)

        ep_list_ret = plugin.get_endpoints(admin_context)
        self.assertItemsEqual(ep_list_ret, [ep_ret1, ep_ret2, ep_ret3])

    def test_create_update_endpoint_name(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep = self._make_ep_dict(ip_mask='0.0.0.0/0',
                                ep_groups=[{'id': epg_ret['id']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        ep_update = self._make_ep_update_dict(name="update_ep_name")
        ep_update_ret = plugin.update_endpoint(admin_context,
                                               ep_ret["id"], ep_update)
        self._consolidate_update_config(ep_update, ep_update_ret)
        self.assertEqual(ep_update["endpoint"], ep_update_ret)

    def test_create_update_endpoint_add_epg(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg1 = self._make_epg_dict()

        epg_ret1 = plugin.create_endpoint_group(
                      admin_context, epg1)
        ep = self._make_ep_dict(ip_mask='0.0.0.0/0',
                                ep_groups=[{'id': epg_ret1['id']}])
        epg2 = self._make_epg_dict(name="test_epg2")

        epg_ret2 = plugin.create_endpoint_group(
                      admin_context, epg2)
        ep_ret = plugin.create_endpoint(admin_context, ep)
        ep_update = self._make_ep_update_dict(add_ep_groups=
                                              [{"id": epg_ret2['id']}])
        ep_update_ret = plugin.update_endpoint(admin_context,
                                               ep_ret["id"], ep_update)
        self._consolidate_update_config(ep_update, ep_update_ret)
        self.assertEqual(ep_update["endpoint"], ep_update_ret)

    def test_create_update_endpoint_remove_epg(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg1 = self._make_epg_dict()

        epg_ret1 = plugin.create_endpoint_group(
                      admin_context, epg1)
        ep = self._make_ep_dict(ip_mask='0.0.0.0/0',
                                ep_groups=[{'id': epg_ret1['id']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        ep_update = self._make_ep_update_dict(remove_ep_groups=
                                              [{"id": epg_ret1['id']}])
        ep_update_ret = plugin.update_endpoint(admin_context,
                                               ep_ret["id"], ep_update)
        self._consolidate_update_config(ep_update, ep_update_ret)
        self.assertEqual(ep_update["endpoint"], ep_update_ret)

    def test_create_update_endpoint_add_epg_name(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg1 = self._make_epg_dict()

        epg_ret1 = plugin.create_endpoint_group(
                      admin_context, epg1)
        ep = self._make_ep_dict(ip_mask='0.0.0.0/0',
                                ep_groups=[{'id': epg_ret1['id']}])
        epg2 = self._make_epg_dict(name="test_epg2")

        epg_ret2 = plugin.create_endpoint_group(
                      admin_context, epg2)
        ep_ret = plugin.create_endpoint(admin_context, ep)
        ep_update = self._make_ep_update_dict(add_ep_groups=
                                              [{"id": epg_ret2['name']}])
        ep_update_ret = plugin.update_endpoint(admin_context,
                                               ep_ret["id"], ep_update)
        self._consolidate_update_config(ep_update, ep_update_ret)
        self.assertEqual(ep_update["endpoint"], ep_update_ret)

    def test_create_update_endpoint_remove_epg_name(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg1 = self._make_epg_dict()

        epg_ret1 = plugin.create_endpoint_group(
                      admin_context, epg1)
        ep = self._make_ep_dict(ip_mask='0.0.0.0/0',
                                ep_groups=[{'id': epg_ret1['id']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        ep_update = self._make_ep_update_dict(remove_ep_groups=
                                              [{"id": epg_ret1['name']}])
        ep_update_ret = plugin.update_endpoint(admin_context,
                                               ep_ret["id"], ep_update)
        self._consolidate_update_config(ep_update, ep_update_ret)
        self.assertEqual(ep_update["endpoint"], ep_update_ret)

    def test_create_update_endpoint_add_duplicate_epg(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg1 = self._make_epg_dict()

        epg_ret1 = plugin.create_endpoint_group(
                      admin_context, epg1)
        ep = self._make_ep_dict(ip_mask='0.0.0.0/0',
                                ep_groups=[{'id': epg_ret1['id']}])
        epg2 = self._make_epg_dict(name="test_epg2")

        epg_ret2 = plugin.create_endpoint_group(
                      admin_context, epg2)
        ep_ret = plugin.create_endpoint(admin_context, ep)
        ep_update = self._make_ep_update_dict(add_ep_groups=
                                              [{"id": epg_ret2['id']},
                                              {"id": epg_ret2['id']}])
        self.assertRaises(policy_excep.DuplicateEndpointGroup,
                          plugin.update_endpoint, admin_context,
                          ep_ret["id"], ep_update)

    def test_create_update_endpoint_remove_duplicate_epg(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg1 = self._make_epg_dict()

        epg_ret1 = plugin.create_endpoint_group(
                      admin_context, epg1)
        ep = self._make_ep_dict(ip_mask='0.0.0.0/0',
                                ep_groups=[{'id': epg_ret1['id']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        ep_update = self._make_ep_update_dict(remove_ep_groups=
                                              [{"id": epg_ret1['id']},
                                              {"id": epg_ret1['id']}])
        self.assertRaises(policy_excep.DuplicateEndpointGroup,
                          plugin.update_endpoint, admin_context,
                          ep_ret["id"], ep_update)

    def test_create_update_endpoint_add_remove_dup_epg(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg1 = self._make_epg_dict()

        epg_ret1 = plugin.create_endpoint_group(
                      admin_context, epg1)
        ep = self._make_ep_dict(ip_mask='0.0.0.0/0',
                                ep_groups=[{'id': epg_ret1['id']}])
        epg2 = self._make_epg_dict(name="test_epg2")

        epg_ret2 = plugin.create_endpoint_group(
                      admin_context, epg2)
        ep_ret = plugin.create_endpoint(admin_context, ep)
        ep_update = self._make_ep_update_dict(add_ep_groups=
                                              [{"id": epg_ret2['id']}],
                                              remove_ep_groups=
                                              [{"id": epg_ret2["id"]}])
        self.assertRaises(policy_excep.DuplicateEndpointGroup,
                          plugin.update_endpoint, admin_context,
                          ep_ret["id"], ep_update)

    def test_create_update_endpoint_add_sg(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg1 = self._make_epg_dict()

        epg_ret1 = plugin.create_endpoint_group(
                      admin_context, epg1)
        ep = self._make_ep_dict(ip_mask='0.0.0.0/0',
                                ep_groups=[{'id': epg_ret1['id']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        sg = self._fake_sg()

        sg_ret = plugin.create_security_group(admin_context, sg)
        ep_update = self._make_ep_update_dict(add_ep_groups=
                                              [{"id": sg_ret['id']}])
        ep_update_ret = plugin.update_endpoint(admin_context, ep_ret["id"],
                                               ep_update)
        self._consolidate_update_config(ep_update, ep_update_ret)
        self.assertEqual(ep_update["endpoint"], ep_update_ret)

    def test_create_update_endpoint_remove_sg(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg1 = self._make_epg_dict()

        epg_ret1 = plugin.create_endpoint_group(
                      admin_context, epg1)
        ep = self._make_ep_dict(ip_mask='0.0.0.0/0',
                                ep_groups=[{'id': epg_ret1['id']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        sg = self._fake_sg()

        sg_ret = plugin.create_security_group(admin_context, sg)
        ep_update = self._make_ep_update_dict(remove_ep_groups=
                                              [{"id": sg_ret['id']}])
        ep_update_ret = plugin.update_endpoint(admin_context, ep_ret["id"],
                                               ep_update)
        self._consolidate_update_config(ep_update, ep_update_ret)
        self.assertEqual(ep_update["endpoint"], ep_update_ret)

    def test_create_update_endpoint_delete_port_bw(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        tenant_context = context.Context('', 'not_admin')

        network = self._fake_network()
        network["network"]["tenant_id"] = tenant_context.tenant_id
        network_ret = plugin.create_network(tenant_context, network)

        port = self._fake_port(network_ret["id"])
        port_ret = plugin.create_port(
                      admin_context, port)

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep = self._make_ep_dict(port=port_ret["id"],
                                ep_groups=[{'id': epg_ret['id']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        plugin.delete_port(admin_context, port_ret["id"])
        ep_update = self._make_ep_update_dict(name="update_ep_name")
        self.assertRaises(ext_ep.NoEndpointFound,
                          plugin.update_endpoint, admin_context,
                          ep_ret["id"], ep_update)

    def test_create_update_get_endpoint_name(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep = self._make_ep_dict(ip_mask='0.0.0.0/0',
                                ep_groups=[{'id': epg_ret['id']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        ep_update = self._make_ep_update_dict(name="update_ep_name")
        ep_update_ret = plugin.update_endpoint(admin_context,
                                               ep_ret["id"], ep_update)
        ep_get_ret = plugin.get_endpoint(admin_context, ep_ret["id"])
        self.assertEqualUpdate(ep_get_ret, ep_update_ret)

    def test_create_update_get_endpoint_add_epg(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg1 = self._make_epg_dict()

        epg_ret1 = plugin.create_endpoint_group(
                      admin_context, epg1)
        ep = self._make_ep_dict(ip_mask='0.0.0.0/0',
                                ep_groups=[{'id': epg_ret1['id']}])
        epg2 = self._make_epg_dict(name="test_epg2")

        epg_ret2 = plugin.create_endpoint_group(
                      admin_context, epg2)
        ep_ret = plugin.create_endpoint(admin_context, ep)
        ep_update = self._make_ep_update_dict(add_ep_groups=
                                              [{"id": epg_ret2['id']}])
        ep_update_ret = plugin.update_endpoint(admin_context,
                                               ep_ret["id"], ep_update)
        ep_get_ret = plugin.get_endpoint(admin_context, ep_ret["id"])
        self.assertEqualUpdate(ep_get_ret, ep_update_ret)

    def test_create_endpoint_with_label(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        epg = self._make_epg_dict()

        epg_ret = plugin.create_endpoint_group(
                      admin_context, epg)
        ep = self._make_ep_dict(label='wire',
                                ep_groups=[{'id': epg_ret['id']}])
        ep_ret = plugin.create_endpoint(admin_context, ep)
        ep["endpoint"]["id"] = ep_ret["id"]
        self.assertEqual(ep_ret, ep["endpoint"])

    def _make_epg_dict(self, name="test_name"):
        return {"endpoint_group": {
                   "tenant_id": "test_tenant",
                   "name": name,
                   "description": "test_description"}}

    def _make_ep_dict(self, name="test_endpoint", port=None,
                      ip_mask=None, ip_port=None, ep_groups=[], label=None):
        return {"endpoint": {
                    "name": name,
                    "tenant_id": "test_tenant",
                    "port_id": port,
                    "ip_mask": ip_mask,
                    "ip_port": ip_port,
                    "label": label,
                    "ep_groups": ep_groups}}

    def _make_ep_update_dict(self, name="test_endpoint",
                             add_ep_groups=[], remove_ep_groups=[]):
        return {"endpoint": {
                   "tenant_id": "test_tenant",
                   "name": name,
                   "add_endpoint_groups": add_ep_groups,
                   "remove_endpoint_groups": remove_ep_groups}}

    def _fake_network(self):
        data = {'network': {'name': "fake_network",
                            'admin_state_up': False,
                            'shared': False,
                            'router:external': [],
                            'provider:network_type': None,
                            'provider:segmentation_id': None,
                            'provider:physical_network': None}}
        return data

    def _fake_subnet(self, net_id):
        allocation_pools = [{'start': '10.0.0.2',
                             'end': '10.0.0.254'}]
        return {'subnet': {'name': "fake_subnet",
                           'network_id': net_id,
                           'gateway_ip': '10.0.0.1',
                           'dns_nameservers': ['10.0.0.2'],
                           'host_routes': [],
                           'cidr': '10.0.0.0/24',
                           'allocation_pools': allocation_pools,
                           'enable_dhcp': True,
                           'ip_version': 4}}

    def _fake_port(self, net_id, security_groups = [],
                   device_owner="compute:nova"):
        return {'port': {'name': 'fake_port',
                         'network_id': net_id,
                         'mac_address': attributes.ATTR_NOT_SPECIFIED,
                         'fixed_ips': attributes.ATTR_NOT_SPECIFIED,
                         'admin_state_up': True,
                         'device_id': 'fake_device_id',
                         'device_owner': device_owner,
                         'tenant_id': "fake_tenant",
                         'security_group': security_groups}}

    def _fake_sg(self):
        return {"security_group": {"name": "fake-sg",
                                   "description": "sample-description",
                                   "tenant_id": "test-tenant"}}

    def assertEqualUpdate(self, ep_ref, ep_actual):
        def make_ep_dict(ep_db):
            if "add_endpoint_groups" in ep_db:
                del ep_db["add_endpoint_groups"]
            if "remove_endpoint_groups" in ep_db:
                del ep_db["remove_endpoint_groups"]
            return ep_db

        ep_ref = make_ep_dict(ep_ref)
        ep_actual = make_ep_dict(ep_actual)
        return self.assertItemsEqual(ep_ref, ep_actual)

    def _consolidate_update_config(self, ep_update, ep_update_ret):
        ep_update["endpoint"]["id"] = ep_update_ret["id"]
        ep_update["endpoint"]["ep_groups"] = ep_update_ret["ep_groups"]
        ep_update["endpoint"]["ip_port"] = ep_update_ret["ip_port"]
        ep_update["endpoint"]["ip_mask"] = ep_update_ret["ip_mask"]
        ep_update["endpoint"]["port_id"] = ep_update_ret["port_id"]
        ep_update["endpoint"]["label"] = ep_update_ret["label"]
